/* this is mainly for faking or replacing functions and variables from the
   FreeCiv source tree that
   1) aren't needed for the map editor, but are needed by functions
      which live in modules that _are_ needed
   2) are needed but come from civserver.c which also contains a main()
*/

#include "civ.h"
#include "tiles.h"

#include <registry.h>
#include <game.h>
#include <map.h>
#include <gamehand.h>
#include <shared.h>
#include <spaceship.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

int is_server=1;
int rand_init=0;
char metaserver_info_line[255];
enum server_states server_state;
char *tile_set_dir=NULL;


/***************************************************************************
figure out the correct scenario type
***************************************************************************/
void set_scenario(void)
 {
  int i;
  /* game.scenario: 0=normal savegame, 1=everything but players,
     2=just tile map and startpositions, 3=just tile map */
  if(game.scenario==0)
    game.scenario=1;
  else if(game.scenario==3)
    game.scenario=2;
  /* default to no start positions if at least 1 is missing */
  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x==0) &&
       (map.start_positions[i].y==0))
      game.scenario=3;
 } /* set_scenario() */


/***************************************************************************
load a save game
***************************************************************************/
void load_game(char *load_filename)
 {
  int i,j;
  struct section_file file;
  struct tile *ptile;

  if(load_filename!=NULL)
   {
    if(map.tiles!=NULL)
      free(map.tiles);
    map.tiles=NULL;
    game_init();

    if(!section_file_load(&file, load_filename))
     {
      fprintf(stderr, "Couldn't load savefile: %s", load_filename);
      return;
     }

    game.scenario=game_load(&file);
    map.generator=0;
    game.nplayers=0;

    /* delete tiles.known, to get type 1 scenarios to work */
    for(i=0;i<map.xsize;i++)
      for(j=0;j<map.ysize;j++)
{        ptile=map_get_tile(i,j);
         if(ptile==NULL)
          printf("erro!\n");
         else ptile->known=0; }

    set_scenario();
   }
 } /* load_game */


/***************************************************************************
save a save game
***************************************************************************/
void save_game(char *save_filename)
{
  struct section_file file;

  server_state=PRE_GAME_STATE;
  update_map_settings();
  map.generator=0;

  section_file_init(&file);
  game_save(&file);

  if(!section_file_save(&file, save_filename))
    fprintf(stderr,"Failed saving %s\n", save_filename);
  else
    fprintf(stderr,"Saved %s\n", save_filename);

  section_file_free(&file);
} /* save_game() */


/*****************************************************************************
complete the given filename
*****************************************************************************/
char *tilefilename(char *name)
{
  char *datadir=datafilename("");
  static char filename[256];
  struct stat foo;

  /* first search tileset directory */
  if(tile_set_dir)  {
    strcpy(filename,datadir);
    strcat(filename,tile_set_dir);
    if(filename[strlen(filename)-1]!='/') strcat(filename,"/");
    strcat(filename,name);
    if(!stat(filename,&foo)) return filename;
  }

  /* wasn't in tileset directory or no tileset specified, use default */
  sprintf(filename,"%sdefault/%s",datadir,name);
  return filename;
}


/*****************************************************************************
fake functions from here on
*****************************************************************************/
/* from server/civserver */
void dealloc_id(int id)
 {
   return;
 }

/* from server/plrhand */
void player_save(struct player *plr, int plrno, struct section_file *file)
 {
  return;
 }

/* from server/plrhand */
void player_load(struct player *plr, int plrno, struct section_file *file)
 {
  return;
 }

/* from server/plrhand */
void notify_player(struct player *pplayer, char *format, ...)
 {
  return;
 }

/* from server/cityturn */
int city_refresh(struct city *pcity)
 {
  return 0;
 }

/* from server/cityhand */
void send_city_info(struct player *dest, struct city *pcity, int dosend)
 {
  return;
 }

/* from server/unitfunc */
void create_unit(struct player *pplayer, int x, int y, enum unit_type_id type)
 {
  return;
 }

/* from server/unitfunc */
void send_unit_info(struct player *dest, struct unit *punit, int dosend)
 {
  return;
 }

/* from common/spaceship.c */
void spaceship_init(struct player_spaceship *ship)
 {
  return;
 }
