/* miscellaneous drawing stuff: init, toggles, and unit drawing */
#include "draw.h"
#include "tiles.h"
#include "tools.h"
#include "toolfunc.h"
#include "civ.h"

#include "mapview.h"

#include <unit.h>
#include <map.h>
/*
#include <genlist.h>
*/

#include <stdio.h>
#include <string.h>

gboolean is_solid_bg=FALSE;
gboolean is_startpos_units=TRUE;
gboolean off_screen=TRUE;
gint ROAD_TILES=32768;
gint RAIL_TILES=0;
gint FLAG_TILES=0;
GtkWidget *popup;

void draw_pixmap(GtkWidget *, gint unit, gint race, gint x, gint y);


/*****************************************************************************
initialize the pixmaps and graphics contexts
*****************************************************************************/
void my_init(GtkWidget *widget)
 {
  /* get white and use it as transparent color. */
  gdk_color_white(gdk_window_get_colormap(widget->window), &white);
  pix_tiles=gdk_pixmap_create_from_xpm(widget->window, &mask_tiles,
				  &white, tilefilename("tiles.xpm"));
  pix_units=gdk_pixmap_create_from_xpm(widget->window, &mask_units,
				  &white, tilefilename("units.xpm"));
  pix_roads=gdk_pixmap_create_from_xpm(widget->window, &mask_roads,
				  &white, tilefilename("roads.xpm"));
  pix_flags=gdk_pixmap_create_from_xpm(widget->window, &mask_flags,
				  &white, tilefilename("flags.xpm"));
  RAIL_TILES=ROAD_TILES+16*2;

  /* get the size of the tiles */
  gdk_window_get_size(pix_tiles,&TILE_WIDTH,&TILE_HEIGHT);
  TILE_WIDTH/=20;TILE_HEIGHT/=18;

  /* copy the masks into the gc */
  gc_tiles=gdk_gc_new(widget->window);
  gdk_gc_set_clip_mask(gc_tiles, mask_tiles);
  gc_roads=gdk_gc_new(widget->window);
  gdk_gc_set_clip_mask(gc_roads, mask_roads);
  gc_units=gdk_gc_new(widget->window);
  gdk_gc_set_clip_mask(gc_units, mask_units);
  gc_flags=gdk_gc_new(widget->window);
  gdk_gc_set_clip_mask(gc_flags, mask_flags);

 } /* my_init() */


/*****************************************************************************
draw a unit: first draw the appropriate flag, then the unit itself
*****************************************************************************/
void draw_pixmap(GtkWidget *widget, gint unit, gint race, gint x, gint y)
 {
  gint tmpX;
  gint tmpY;

  tmpX=(race+FLAG_TILES)%14;
  tmpY=(race+FLAG_TILES)/14;

  draw_tile(widget, x, y, NULL);
  /* draw the flag */
  if(!is_solid_bg)
   {
    gdk_gc_set_clip_origin(gc_flags,(x-tmpX)*TILE_WIDTH, (y-tmpY)*TILE_HEIGHT);
    gdk_draw_pixmap(pix_screen, gc_flags, pix_flags,
		    tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		    x*TILE_WIDTH, y*TILE_HEIGHT,
		    TILE_WIDTH, TILE_HEIGHT);
   }
  else
    gdk_draw_pixmap(pix_screen, widget->style->white_gc, pix_flags,
		    tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		    x*TILE_WIDTH, y*TILE_HEIGHT,
		    TILE_WIDTH, TILE_HEIGHT);

  /* draw the unit */
  if(is_startpos_units)
   {
    tmpX=unit_types[unit].graphics%20;
    tmpY=unit_types[unit].graphics/20;
    gdk_gc_set_clip_origin(gc_units,(x-tmpX)*TILE_WIDTH, (y-tmpY)*TILE_HEIGHT);
    gdk_draw_pixmap(pix_screen, gc_units, pix_units,
		    tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		    x*TILE_WIDTH, y*TILE_HEIGHT,
		    TILE_WIDTH, TILE_HEIGHT);
   }
 } /* draw_pixmap() */


/*****************************************************************************
draw the units: first draw settlers on all starting positions (then all units)
*****************************************************************************/
void draw_units(GtkWidget *widget, gint x, gint y, gint w, gint h)
{
/*  struct genlist_iterator iterator;
  struct unit *punit; */
  gint i;

  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x!=0) ||
       (map.start_positions[i].y!=0))
      draw_pixmap(widget,U_SETTLERS,i,
                  map.start_positions[i].x, map.start_positions[i].y);

/*
  for(i=0; i<game.nplayers; i++)
    {
     genlist_iterator_init(&iterator,
                          (struct genlist *) &game.players[i].units, 0);
     for(; ITERATOR_PTR(iterator); ITERATOR_NEXT(iterator))
      {
	punit=(struct unit *)ITERATOR_PTR(iterator);
	if((punit->x >= x) &&(punit->y >= y) &&
	   (punit->x <=(x+w+1)) &&(punit->y <=(y+h+1)))
	 {
	  draw_pixmap(widget, punit->type, game.players[punit->owner].race,
		       punit->x, punit->y);
	 }
      }
    }
*/

 } /* draw_units() */


/****************************************************************************
drawing area expose event handler
****************************************************************************/
guint expose(GtkWidget *widget, GdkEventExpose *ev)
 {
  gint x,y;

  if(!off_screen)
   {
    for(x=ev->area.x/TILE_WIDTH-1;x<=(ev->area.x+ev->area.width)/TILE_WIDTH+1;x++)
      for(y=ev->area.y/TILE_HEIGHT-1;y<=(ev->area.y+ev->area.height)/TILE_HEIGHT+1;y++)
        draw_tile(widget, x, y, NULL);
   }
  draw_units(widget, ev->area.x/TILE_WIDTH-1, ev->area.y/TILE_HEIGHT-1,
	     ev->area.width/TILE_WIDTH+1, ev->area.height/TILE_HEIGHT+1);
  if(off_screen)
   {
    gdk_draw_pixmap(widget->window, widget->style->white_gc, pix_screen,
		    ev->area.x, ev->area.y,
		    ev->area.x, ev->area.y, ev->area.width, ev->area.height);
   }
  return FALSE;
 } /* expose() */


/*****************************************************************************
race label expose event handler
*****************************************************************************/
guint expose_racelabel(GtkWidget *widget, GdkEventExpose *ev)
 {
  gint tmpX, tmpY;

  tmpX=(country+FLAG_TILES)%14;
  tmpY=(country+FLAG_TILES)/14;

  gdk_window_clear(widget->window);
  gdk_gc_set_clip_origin(gc_flags,(-tmpX)*TILE_WIDTH,(-tmpY)*TILE_HEIGHT);
  gdk_draw_pixmap(widget->window, gc_flags, pix_flags,
		  tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,0,0,
		  TILE_WIDTH, TILE_HEIGHT);
  return FALSE;
 } /* expose_racelabel() */


/*****************************************************************************
redraw the whole map screen by generating a fake expose event
*****************************************************************************/
void refresh_screen()
{
  GdkEventExpose tmp;

  /* when there is no off-screen pixmap, there's nothing to do here */
  if(!off_screen)
    return;

  tmp.area.x=0;
  tmp.area.y=0;
  tmp.area.width=map.xsize*TILE_WIDTH;
  tmp.area.height=map.ysize*TILE_HEIGHT;

  expose(GTK_WIDGET(da), &tmp);
} /* refresh_screen() */


/*****************************************************************************
destroy the requested widget
*****************************************************************************/
void destroy_me_callback(GtkWidget *w, gpointer data)
{
  gtk_widget_destroy(w);
} /* destroy_me_callback() */


/*****************************************************************************
pop up information about the current tile
*****************************************************************************/
void popitup(GdkEventButton *ev, gint x, gint y)
 {
  char s[64];
  struct tile *ptile;
  GtkWidget *item;

  x/=TILE_WIDTH;
  y/=TILE_HEIGHT;

  popup=gtk_menu_new();
  gtk_signal_connect_after(GTK_OBJECT(popup),"hide",
			   GTK_SIGNAL_FUNC(destroy_me_callback),NULL);
  gtk_signal_connect_after(GTK_OBJECT(popup),"button_release_event",
			   GTK_SIGNAL_FUNC(destroy_me_callback),NULL);

  ptile=map_get_tile(x, y);
  sprintf(s, "Terrain: %s", map_get_tile_info_text(x, y));

  item=gtk_menu_item_new_with_label(s);
  gtk_widget_set_sensitive(item,FALSE);
  gtk_menu_append(GTK_MENU(popup),item);

  if(ptile->special&S_HUT)
   {
    item=gtk_menu_item_new_with_label("Minor Tribe Village");
    gtk_widget_set_sensitive(item,FALSE);
    gtk_menu_append(GTK_MENU(popup),item);
   }

/*
  if((pcity=map_get_city(xtile, ytile))) {
    sprintf(s, "City: %s(%s)", pcity->name,
            get_race_name(game.players[pcity->owner].race));
    item=gtk_menu_item_new_with_label(s);
    gtk_widget_set_sensitive(item,FALSE);
    gtk_menu_append(GTK_MENU(popup),item);
  }
*/

 if(ptile->special&S_ROAD || ptile->special&S_IRRIGATION ||
     ptile->special&S_RAILROAD || ptile->special&S_MINE ||
     ptile->special&S_FORTRESS)
  {
   sprintf(s, "Infrastructure: ");
   if(ptile->special&S_ROAD)
     strcat(s, "Road/");
   if(ptile->special&S_RAILROAD)
     strcat(s, "Railroad/");
   if(ptile->special&S_IRRIGATION)
     strcat(s, "Irrigation/");
   if(ptile->special&S_MINE)
     strcat(s, "Mine/");
   if(ptile->special&S_FORTRESS)
     strcat(s, "Fortress/");
   *(s+strlen(s)-1)='\0';
   item=gtk_menu_item_new_with_label(s);
   gtk_widget_set_sensitive(item,FALSE);
   gtk_menu_append(GTK_MENU(popup),item);
  }

  gtk_widget_show_all(popup);
  gtk_menu_popup(GTK_MENU(popup),NULL,NULL,NULL,NULL,ev->button,ev->time);

 } /* popitup() */


/*****************************************************************************
toggle the solid unit background on and off
*****************************************************************************/
void solid_bg(void)
 {
  is_solid_bg=(!is_solid_bg);
  draw_units(GTK_WIDGET(da), 0, 0, map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT);
  refresh_screen();
 } /* solid_bg() */


/*****************************************************************************
toggle the drawing of units on and off
*****************************************************************************/
void startpos_units(void)
 {
  is_startpos_units=(!is_startpos_units);
  draw_units(GTK_WIDGET(da), 0, 0, map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT);
  refresh_screen();
 } /* startpos_units() */
