#ifndef __DRAW_H__
#define __DRAW_H__


#include "civedit.h"
/* Normally, we use an off-screen pixmap for drawing stuff on, and then just
   copy the relevant areas from the pixmap onto the screen when we need to
   refresh an area or re-draw everything. This immensely speeds up scrolling
   the map area and processing expose events.

   OTOH, with large (>= 100x100) maps, decently large tile sets
   (tile size >= 30x30) and higher (>= 16 bit) color depths, this off-screen
   pixmap can get rather large:

   memory needed=(depth in bytes)*TILE_HEIGHT*TILE_WIDTH*map.xsize*map.ysize;

   on a 32-bit display with the default tiles and a 100x100 map, that's
   already 4*30*30*100*100=36MB(!) which can grow to 160 MB with the engels
   tiles (45x45) and the maximum map size of 200x100.

   For this reason, off-screen drawing gets switched off when
   TILE_HEIGHT*TILE_WIDTH*map.xsize*map.ysize exceeds the value below.
   The default for this is 10MB.
*/

#define OFF_SCREEN_MAX_SIZE 10000000


GdkPixmap *pix_units, *pix_tiles, *pix_roads, *pix_flags;
GdkBitmap *mask_units, *mask_tiles, *mask_roads, *mask_flags;
GdkGC *gc_units, *gc_tiles, *gc_roads, *gc_flags;
GdkColor white;
GdkDrawable *pix_screen;
extern gboolean off_screen;

void my_init(GtkWidget *widget);
void draw_units(GtkWidget *widget, gint x, gint y, gint width, gint height);
guint expose(GtkWidget *widget, GdkEventExpose *ev);
guint expose_racelabel(GtkWidget *widget, GdkEventExpose *ev);
void refresh_screen(void);
void popitup(GdkEventButton *ev, gint x, gint y);
void solid_bg(void);
void startpos_units(void);


#endif /* __DRAW_H__ */
