#include "main.h"
#include "civ.h"
#include "tiles.h"
#include "draw.h"
#include "tools.h"
#include "toolfunc.h"
#include "overview.h"
#include "menu.h"

#include <map.h>
/*
#include <genlist.h>
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


GtkWidget *terrainlabel;
GtkWidget *racelabel;
GtkWidget *coordlabel;
GtkTooltips *tooltips;
GtkWidget *scroll;

gint xdiff=0, ydiff=0;
gboolean pressed=FALSE;

void create_main_window(void);
/* drawing area event handlers */
void press(GtkWidget *widget, GdkEventButton *ev);
void release(GtkWidget *widget, GdkEventButton *ev);
void motion(GtkWidget *widget, GdkEventMotion *ev);

/*****************************************************************************
main() - need I say more? parses the command line and sets up the windows
*****************************************************************************/
int main(int argc, char **argv)
 {
  gint i=1, h=0;

  gtk_init(&argc, &argv);

/*
  genlist_init(&city_to_draw_list);
  city_font=gdk_font_load(DEFAULT_FONT);
*/

  while(i<argc)
   {
    if(!strcmp("-t", argv[i]) || !strcmp("--tiles", argv[i]))
     {
      if(++i<argc)
        tile_set_dir=argv[i];
      else
       {
        fprintf(stderr, "Error: tileset directory not specified.\n");
        h=1;
        break;
       }
     }
    else if(!strcmp("-f", argv[i]) || !strcmp("--file", argv[i]))
     {
      if(++i<argc)
       {
        filename=(char *)malloc(256*sizeof(char));
        strcpy(filename,argv[i]);
       }
      else
       {
        fprintf(stderr, "Error: scenario/save game not specified.\n");
        h=1;
        break;
       }
     }
    else if(!strcmp("-h", argv[i]) || !strcmp("--help", argv[i]))
     {
      h=1;
      break;
     }
    else
     {
      fprintf(stderr, "Error: unknown option '%s'\n", argv[i]);
      h=1;
      break;
     }
    i++;
   }

  if(h)
   {
    fprintf(stderr, "This is CivWorld, the Freeciv map editor.\n");
    fprintf(stderr, "  -h, --help\t\t\tPrint a summary of the options, then exit.\n");
    fprintf(stderr, "  -f, --file <save game>\tSpecify a save game or scenario to load,\n");
    fprintf(stderr, "  -t, --tiles <tileset dir>\tSpecify a subdirectory of the FreeCiv data\n");
    fprintf(stderr, "\t\t\t\tdirectory in which to look for a tile set.\n");
    exit(0);
   }

  create_main_window();
  create_tools_palette();
  create_ov();

  if(filename!=NULL)
    open_ok(NULL, NULL);
  else
    draw_everything(FALSE);

  gtk_main();

  return 0;
 } /* main() */


/*****************************************************************************
create and setup the main window and everything around it
*****************************************************************************/
void create_main_window(void)
 {
  GtkWidget *box;
  GtkWidget *hbox;
  GtkWidget *templabel;
  GtkAdjustment *adj;

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(win), "FreeWorld - Cause the world should be free");
  gtk_window_set_default_size(GTK_WINDOW(win), 600,401);

  box = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(win), box);

  create_menus(box);

  scroll=gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  da=(GtkDrawingArea *) gtk_drawing_area_new();
  gtk_drawing_area_size(da, map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll),
					GTK_WIDGET(da));
  gtk_signal_connect(GTK_OBJECT(da), "expose_event",
		      GTK_SIGNAL_FUNC(expose), NULL);
  gtk_signal_connect(GTK_OBJECT(da), "motion_notify_event",
		      GTK_SIGNAL_FUNC(motion), NULL);
  gtk_signal_connect(GTK_OBJECT(da), "button_press_event",
		      GTK_SIGNAL_FUNC(press), NULL);
  gtk_signal_connect(GTK_OBJECT(da), "button_release_event",
		      GTK_SIGNAL_FUNC(release), NULL);
  gtk_signal_connect(GTK_OBJECT(win), "delete_event",
		      GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
  gtk_widget_set_events(GTK_WIDGET(da), GDK_BUTTON_PRESS_MASK
			 | GDK_KEY_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
			 | GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK);

  gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(scroll), TRUE, TRUE, 0);

  hbox=gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 3);
  gtk_widget_show(hbox);

  hbox=gtk_hbox_new(FALSE, 0);

  gtk_box_pack_end(GTK_BOX(box), GTK_WIDGET(hbox), FALSE, FALSE, 3);
  templabel=gtk_label_new("Terrain: ");
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(templabel), FALSE, FALSE, 5);
  gtk_widget_show(GTK_WIDGET(templabel));

  terrainlabel=gtk_label_new("Arctic ");
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(terrainlabel), FALSE, FALSE, 5);
  gtk_widget_show(GTK_WIDGET(terrainlabel));

  templabel=gtk_label_new("Race: ");
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(templabel), FALSE, FALSE, 5);
  gtk_widget_show(GTK_WIDGET(templabel));

  racelabel=gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(racelabel),TILE_WIDTH, TILE_HEIGHT);
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(racelabel), FALSE, FALSE, 5);
  gtk_signal_connect(GTK_OBJECT(racelabel), "expose_event",
		      GTK_SIGNAL_FUNC(expose_racelabel), NULL);
  gtk_widget_show(GTK_WIDGET(racelabel));

  coordlabel=gtk_label_new("(   ,   )");
  gtk_box_pack_end(GTK_BOX(hbox), GTK_WIDGET(coordlabel), FALSE, FALSE, 5);
  gtk_widget_show(GTK_WIDGET(coordlabel));

  templabel=gtk_label_new("Map Coordinates:");
  gtk_box_pack_end(GTK_BOX(hbox), GTK_WIDGET(templabel), FALSE, FALSE, 5);
  gtk_widget_show(GTK_WIDGET(templabel));

  gtk_widget_show(GTK_WIDGET(da));
  gtk_widget_show(GTK_WIDGET(scroll));
  gtk_widget_show(GTK_WIDGET(hbox));
  gtk_widget_show(GTK_WIDGET(box));
  gtk_widget_show(GTK_WIDGET(win));

  my_init(GTK_WIDGET(da));
  create_new_map(MAP_DEFAULT_WIDTH,MAP_DEFAULT_HEIGHT);

  gdk_window_get_size(win->window, &xdiff, &ydiff);
  adj=gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scroll));
  xdiff-=2*adj->page_increment;
  adj=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scroll));
  ydiff-=2*adj->page_increment;

 } /* create_main_window() */


/*****************************************************************************
return the total width/height of everything surrounding the viewport
*****************************************************************************/
void get_deco_size(gint *x, gint* y)
 {
  *x=xdiff;
  *y=ydiff;
 } /* get_deco_size() */


/*****************************************************************************
set the text for the terrain label
*****************************************************************************/
void set_terrain_label(const char *label)
 {
  if(terrainlabel!=NULL)
    gtk_label_set_text(GTK_LABEL(terrainlabel),label);
 } /* set_terrain_label() */


/*****************************************************************************
redraw the race label by generating a fake expose event
*****************************************************************************/
void refresh_racelabel()
 {
  GdkEventExpose tmp;

  tmp.area.x=0;
  tmp.area.y=0;
  tmp.area.width=TILE_WIDTH;
  tmp.area.height=TILE_HEIGHT;

  expose_racelabel(GTK_WIDGET(racelabel), &tmp);
 } /* refresh_racelabel() */


/*****************************************************************************
set the new values for the coordinates label
*****************************************************************************/
void set_coord_label(const gint x, const gint y)
 {
  char label[10];
  if(coordlabel!=NULL) {
    sprintf(label,"(%3.3i,%3.3i)",x,y);
    gtk_label_set_text(GTK_LABEL(coordlabel),label);
  }
 } /* set_terrain_label() */


/*****************************************************************************
center the map on the given tile
*****************************************************************************/
void center_map(gint x, gint y)
 {
  GtkAdjustment *adj;
  gfloat value;

  adj=gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scroll));
  value=min(max(0,x*TILE_WIDTH-adj->page_size/2),adj->upper-adj->page_size);
  gtk_adjustment_set_value(adj, value);
  gtk_signal_emit_by_name(GTK_OBJECT(adj), "changed");
  adj=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scroll));
  value=min(max(0,y*TILE_HEIGHT-adj->page_size/2),adj->upper-adj->page_size);
  gtk_adjustment_set_value(adj, value);
 } /* center_map() */


/****************************************************************************
mouse button click event
****************************************************************************/
void press(GtkWidget *widget, GdkEventButton *ev)
 {
  switch(ev->button)
   {
    case 1:
      pressed=TRUE;
      if(current_tool->function_press != NULL)
       {
	(current_tool->function_press)(widget, ev->x, ev->y);
       }
    break;
    case 2:
      popitup(ev, ev->x, ev->y);
    break;
    case 3:
      change_race();
      pressed=FALSE;
      clear_sel();
    break;
   }
 } /* press() */


/****************************************************************************
mouse move event
****************************************************************************/
void motion(GtkWidget *widget, GdkEventMotion *ev)
 {
  set_coord_label(ev->x/TILE_WIDTH, ev->y/TILE_HEIGHT);
  if((pressed) && (current_tool->function_press != NULL))
   {
    (current_tool->function_press)(widget, ev->x, ev->y);
   }
 } /* motion() */


/****************************************************************************
mouse button release event
****************************************************************************/
void release(GtkWidget *widget, GdkEventButton *ev)
 {
  switch(ev->button)
   {
    case 1:
      if(pressed && (current_tool->function_release != NULL))
       {
	(current_tool->function_release)(widget, ev->x, ev->y);
       }
    break;
    case 2:
    break;
    case 3:
    break;
   }
  pressed=FALSE;
 } /* release() */
