#include "civedit.h"
#include "overview.h"
#include "main.h"
#include "menu.h"

#include "map.h"

GtkWidget *ov_win=NULL;
GtkWidget *draw=NULL;
GdkGC *ov_gc=NULL;
gboolean ov_shown=TRUE;

GdkColor ov_colors[]=
 {
  { 0, 255<<8, 255<<8, 255<<8 }, /* white:       arctic */
  { 0, 255<<8, 248<<8,   0<<8 }, /* yellow:      desert */
  { 0,   0<<8, 112<<8,   0<<8 }, /* dark green:  forest */
  { 0,   0<<8, 160<<8,   0<<8 }, /* green:       grassland */
  { 0, 192<<8, 160<<8,   0<<8 }, /* brown:       hills */
  { 0,  64<<8, 255<<8,   0<<8 }, /* light green: jungle */
  { 0, 128<<8,  96<<8,  96<<8 }, /* brown/grey:  mountains */
  { 0,   0<<8,   0<<8, 255<<8 }, /* blue         ocean */
  { 0, 192<<8, 192<<8,   0<<8 }, /* brown-yellow:plains */
  { 0,  96<<8,  96<<8, 255<<8 }, /* light blue:  river */
  { 0,   0<<8, 192<<8, 192<<8 }, /* cyan:        swamp */
  { 0, 192<<8, 192<<8, 192<<8 }, /* light grey:  tundra */
  { 0, 255<<8,  64<<8,  64<<8 }  /* light red:   unit */
 };


gint ov_delete(GtkWidget *widget, GdkEvent *ev, gpointer data);


/*****************************************************************************
center the map on the clicked location
*****************************************************************************/
void ov_press(GtkWidget *widget, GdkEventButton *ev)
 {
  center_map(ev->x/2, ev->y/2);
 } /* ov_press() */


/*****************************************************************************
prevent the overview window from being resized
*****************************************************************************/
gint ov_configure(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 {
  if((ev->width!=2*map.xsize) ||(ev->height!=2*map.ysize))
    ov_set_size();
  return TRUE;
 } /* destroyWindow() */


/***************************************************************************
create the overview window
***************************************************************************/
void create_ov(void)
 {
  gboolean i[13];

  ov_win=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(ov_win), "Map Overview");
  gtk_widget_set_usize(GTK_WIDGET(ov_win), MAP_MIN_WIDTH, MAP_MIN_HEIGHT);
  gtk_window_set_transient_for(GTK_WINDOW(ov_win), GTK_WINDOW(win));
  gtk_container_set_border_width(GTK_CONTAINER(ov_win), 0);
  gtk_signal_connect(GTK_OBJECT(ov_win), "delete_event",
		     GTK_SIGNAL_FUNC(ov_delete), ov_win);
  gtk_signal_connect(GTK_OBJECT(ov_win), "configure_event",
		     GTK_SIGNAL_FUNC(ov_configure), ov_win);

  draw=gtk_drawing_area_new();
  gtk_container_add(GTK_CONTAINER(ov_win), draw);
  gtk_signal_connect(GTK_OBJECT(draw), "expose_event",
		     GTK_SIGNAL_FUNC(ov_expose), NULL);
  gtk_signal_connect(GTK_OBJECT(draw), "button_press_event",
		     GTK_SIGNAL_FUNC(ov_press), NULL);
  gtk_widget_set_events(GTK_WIDGET(draw), GDK_BUTTON_PRESS_MASK);
  gtk_widget_show(draw);
  gtk_widget_show(ov_win);
  ov_set_size();

  ov_gc=gdk_gc_new(draw->window);
  gdk_gc_set_function(ov_gc,GDK_COPY);
  gdk_colormap_alloc_colors(gtk_widget_get_colormap(draw), ov_colors, 13,
			    TRUE, FALSE, i);
 } /* create_ov() */


/*****************************************************************************
overview window expose handler
*****************************************************************************/
guint ov_expose(GtkWidget *widget, GdkEventExpose *ev)
 {
  gint x,y;
  for(x=ev->area.x/2;x<=(ev->area.x+ev->area.width)/2+1;x++)
    for(y=ev->area.y/2;y<=(ev->area.y+ev->area.height)/2+1;y++)
      ov_draw_tile(x, y, NULL);

  return FALSE;
 } /* ov_expose()*/


/*****************************************************************************
draw a tile in the overview window: also draw startpositions
*****************************************************************************/
void ov_draw_tile(gint x, gint y, struct genlist *citylist)
 {
  gint ttype;
  gint i;
  GdkColor *color;

  if((x>=map.xsize) || (y>=map.ysize) || (ov_win==NULL))
    return;

  ttype=map_get_terrain(x,y);
  /* set color according to tile type */
  color=&ov_colors[ttype];
  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x==x) &&
       (map.start_positions[i].y==y))
      color=&ov_colors[T_UNKNOWN];

  gdk_gc_set_foreground(ov_gc, color);
  gdk_draw_point(draw->window, ov_gc, 2*x, 2*y);
  gdk_draw_point(draw->window, ov_gc, 2*x+1, 2*y);
  gdk_draw_point(draw->window, ov_gc, 2*x, 2*y+1);
  gdk_draw_point(draw->window, ov_gc, 2*x+1, 2*y+1);
 } /* ov_draw_tile() */


/*****************************************************************************
show the overview window
*****************************************************************************/
void show_ov(void)
 {
   if(ov_shown)
     gtk_widget_hide(GTK_WIDGET(ov_win));
   else
     gtk_widget_show(GTK_WIDGET(ov_win));
   ov_shown=(!ov_shown);
 } /* show_ov() */


/*****************************************************************************
hide the overview window upon wm delete
*****************************************************************************/
gint ov_delete(GtkWidget *widget, GdkEvent *ev, gpointer data)
 {
  toggle_over_off();
  return TRUE;
 } /* destroyWindow() */


/*****************************************************************************
adjust the size of the overview window to the map size
*****************************************************************************/
void ov_set_size(void)
 {
  gtk_drawing_area_size(GTK_DRAWING_AREA(draw), 2*map.xsize, 2*map.ysize);
  gdk_window_resize(ov_win->window, 2*map.xsize, 2*map.ysize);
 } /* ov_set_size() */
