#include "civedit.h"
#include "tiles.h"
#include "draw.h"

#include <map.h>
#include <game.h>

#include <stdio.h>

#define TABLE_ROWS 5

#define NUM_SETTINGS 39
#define SSET_MAX_LEN 16	/* max setting name length (plus nul) */

/* pilfered from server/stdinhand.c */
enum sset_class
 {
  SSET_MAP_SIZE,
  SSET_MAP_GEN,
  SSET_MAP_ADD,
  SSET_PLAYERS,
  SSET_GAME_INIT,
  SSET_RULES,
  SSET_RULES_FLEXIBLE,
  SSET_META,
  SSET_LAST
 };

enum sset_to_client
 {
  SSET_TO_CLIENT, SSET_SERVER_ONLY
 };

struct settings_s
 {
  char *name;
  int *value;
  enum sset_class sclass;
  enum sset_to_client to_client;
  int min_value, max_value, default_value;
  char *short_help;
  GtkWidget *spinner;
 };


/* adopted and adapted from server/stdinhand.c */
struct settings_s settings[] = {
/* Map size parameters: adjustable if we don't yet have a map */
  { "xsize", &map.xsize,
    SSET_MAP_SIZE, SSET_TO_CLIENT,
    MAP_MIN_WIDTH, MAP_MAX_WIDTH, MAP_DEFAULT_WIDTH,
    "Map width in squares", NULL },

  { "ysize", &map.ysize,
    SSET_MAP_SIZE, SSET_TO_CLIENT,
    MAP_MIN_HEIGHT, MAP_MAX_HEIGHT, MAP_DEFAULT_HEIGHT,
    "Map height in squares", NULL },

/* Map generation parameters: once we have a map these are of historical
 * interest only, and cannot be changed.
 */
  { "generator", &map.generator,
    SSET_MAP_GEN, SSET_TO_CLIENT,
    MAP_MIN_GENERATOR, MAP_MAX_GENERATOR, MAP_DEFAULT_GENERATOR,
    "Method used to generate map", NULL },

  { "landmass", &map.landpercent,
    SSET_MAP_GEN, SSET_TO_CLIENT,
    MAP_MIN_LANDMASS, MAP_MAX_LANDMASS, MAP_DEFAULT_LANDMASS,
    "Amount of land vs ocean", NULL },

  { "mountains", &map.mountains,
    SSET_MAP_GEN, SSET_TO_CLIENT,
    MAP_MIN_MOUNTAINS, MAP_MAX_MOUNTAINS, MAP_DEFAULT_MOUNTAINS,
    "Amount of hills/mountains", NULL },

  { "rivers", &map.riverlength,
    SSET_MAP_GEN, SSET_TO_CLIENT,
    MAP_MIN_RIVERS, MAP_MAX_RIVERS, MAP_DEFAULT_RIVERS,
    "Amount of river squares", NULL },

  { "forests", &map.forestsize,
    SSET_MAP_GEN, SSET_TO_CLIENT,
    MAP_MIN_FORESTS, MAP_MAX_FORESTS, MAP_DEFAULT_FORESTS,
    "Amount of forest squares", NULL },

  { "swamps", &map.swampsize,
    SSET_MAP_GEN, SSET_TO_CLIENT,
    MAP_MIN_SWAMPS, MAP_MAX_SWAMPS, MAP_DEFAULT_SWAMPS,
    "Amount of swamp squares", NULL },

  { "deserts", &map.deserts,
    SSET_MAP_GEN, SSET_TO_CLIENT,
    MAP_MIN_DESERTS, MAP_MAX_DESERTS, MAP_DEFAULT_DESERTS,
    "Amount of desert squares", NULL },

  { "seed", &map.seed,
    SSET_MAP_GEN, SSET_SERVER_ONLY,
    MAP_MIN_SEED, MAP_MAX_SEED, MAP_DEFAULT_SEED,
    "Map generation random seed", NULL },

/* Map additional stuff: huts and specials.  randseed also goes here
 * because huts and specials are the first time the randseed gets used (?)
 * These are done when the game starts, so these are historical and
 * fixed after the game has started.
 */
  { "randseed", &game.randseed,
    SSET_MAP_ADD, SSET_SERVER_ONLY,
    GAME_MIN_RANDSEED, GAME_MAX_RANDSEED, GAME_DEFAULT_RANDSEED,
    "General random seed", NULL },

  { "specials", &map.riches,
    SSET_MAP_ADD, SSET_TO_CLIENT,
    MAP_MIN_RICHES, MAP_MAX_RICHES, MAP_DEFAULT_RICHES,
    "Amount of \"special\" resource squares", NULL },

  { "huts", &map.huts,
    SSET_MAP_ADD, SSET_TO_CLIENT,
    MAP_MIN_HUTS, MAP_MAX_HUTS, MAP_DEFAULT_HUTS,
    "Amount of huts (minor tribe villages)", NULL },

/* Options affecting numbers of players and AI players.  These only
 * affect the start of the game and can not be adjusted after that.
 * (Actually, minplayers does also affect reloads: you can't start a
 * reload game until enough players have connected (or are AI).)
 */
  { "minplayers", &game.min_players,
    SSET_PLAYERS, SSET_TO_CLIENT,
    GAME_MIN_MIN_PLAYERS, GAME_MAX_MIN_PLAYERS, GAME_DEFAULT_MIN_PLAYERS,
    "Minimum number of players", NULL },

  { "maxplayers", &game.max_players,
    SSET_PLAYERS, SSET_TO_CLIENT,
    GAME_MIN_MAX_PLAYERS, GAME_MAX_MAX_PLAYERS, GAME_DEFAULT_MAX_PLAYERS,
    "Maximum number of players", NULL },

  { "aifill", &game.aifill,
    SSET_PLAYERS, SSET_TO_CLIENT,
    GAME_MIN_AIFILL, GAME_MAX_AIFILL, GAME_DEFAULT_AIFILL,
    "Number of players to fill to with AI's", NULL },

/* Game initialization parameters (only affect the first start of the game,
 * and not reloads).  Can not be changed after first start of game.
 */
  { "settlers", &game.settlers,
    SSET_GAME_INIT, SSET_TO_CLIENT,
    GAME_MIN_SETTLERS, GAME_MAX_SETTLERS, GAME_DEFAULT_SETTLERS,
    "Number of initial settlers per player", NULL },

  { "explorer", &game.explorer,
    SSET_GAME_INIT, SSET_TO_CLIENT,
    GAME_MIN_EXPLORER, GAME_MAX_EXPLORER, GAME_DEFAULT_EXPLORER,
    "Number of initial explorers per player", NULL },

  { "gold", &game.gold,
    SSET_GAME_INIT, SSET_TO_CLIENT,
    GAME_MIN_GOLD, GAME_MAX_GOLD, GAME_DEFAULT_GOLD,
    "Starting gold per player", NULL },

  { "techlevel", &game.tech,
    SSET_GAME_INIT, SSET_TO_CLIENT,
    GAME_MIN_TECHLEVEL, GAME_MAX_TECHLEVEL, GAME_DEFAULT_TECHLEVEL,
    "Number of initial advances per player", NULL },

/* Various rules: these cannot be changed once the game has started. */
  { "researchspeed", &game.techlevel,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_RESEARCHLEVEL, GAME_MAX_RESEARCHLEVEL, GAME_DEFAULT_RESEARCHLEVEL,
    "Points required to gain a new advance", NULL },

  { "techpenalty", &game.techpenalty,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_TECHPENALTY, GAME_MAX_TECHPENALTY, GAME_DEFAULT_TECHPENALTY,
    "Percentage penalty when changing tech", NULL },

  { "diplcost", &game.diplcost,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_DIPLCOST, GAME_MAX_DIPLCOST, GAME_DEFAULT_DIPLCOST,
    "Penalty when getting tech from treaty", NULL },

  { "conquercost", &game.conquercost,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_CONQUERCOST, GAME_MAX_CONQUERCOST, GAME_DEFAULT_CONQUERCOST,
    "Penalty when getting tech from conquering", NULL },

  { "freecost", &game.freecost,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_FREECOST, GAME_MAX_FREECOST, GAME_DEFAULT_FREECOST,
    "Penalty when getting a free tech", NULL },

  { "railprod", &game.rail_prod,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_RAILPROD, GAME_MAX_RAILPROD, GAME_DEFAULT_RAILPROD,
    "Railroad modifier to shield production", NULL },

  { "railfood", &game.rail_food,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_RAILFOOD, GAME_MAX_RAILFOOD, GAME_DEFAULT_RAILFOOD,
    "Railroad modifier to food production", NULL },

  { "railtrade", &game.rail_trade,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_RAILTRADE, GAME_MAX_RAILTRADE, GAME_DEFAULT_RAILTRADE,
    "Railroad modifier to trade production", NULL },

  { "foodbox", &game.foodbox,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_FOODBOX, GAME_MAX_FOODBOX, GAME_DEFAULT_FOODBOX,
    "Food required for a city to grow", NULL },

  { "aqueductloss", &game.aqueductloss,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_AQUEDUCTLOSS, GAME_MAX_AQUEDUCTLOSS, GAME_DEFAULT_AQUEDUCTLOSS,
    "Percentage food lost when need aqueduct", NULL },

  { "unhappysize", &game.unhappysize,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_UNHAPPYSIZE, GAME_MAX_UNHAPPYSIZE, GAME_DEFAULT_UNHAPPYSIZE,
    "City size before people become unhappy", NULL },

  { "cityfactor", &game.cityfactor,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_CITYFACTOR, GAME_MAX_CITYFACTOR, GAME_DEFAULT_CITYFACTOR,
    "Number of cities for higher unhappiness", NULL },

  { "razechance", &game.razechance,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_RAZECHANCE, GAME_MAX_RAZECHANCE, GAME_DEFAULT_RAZECHANCE,
    "Chance for conquered building destruction", NULL },

  { "civstyle", &game.civstyle,
    SSET_RULES, SSET_TO_CLIENT,
    GAME_MIN_CIVSTYLE, GAME_MAX_CIVSTYLE, GAME_DEFAULT_CIVSTYLE,
    "Style of Civ rules", NULL },

/* Flexible rules: these can be changed after the game has started.
 * Should such flexible rules exist?  diplchance is included here
 * to duplicate its previous behaviour (and note diplchance is only used
 * in the server, so it is "safe" to change).  --dwp
 */
  { "diplchance", &game.diplchance,
    SSET_RULES_FLEXIBLE, SSET_TO_CLIENT,
    GAME_MIN_DIPLCHANCE, GAME_MAX_DIPLCHANCE, GAME_DEFAULT_DIPLCHANCE,
    "Chance (1 in N) for diplomat/spy contests and spy returns", NULL },

/* Meta options: these don't affect the internal rules of the game, but
 * do affect players.  Also options which only produce extra server
 * "output" and don't affect the actual game.
 * ("endyear" is here, and not RULES_FLEXIBLE, because it doesn't
 * affect what happens in the game, it just determines when the
 * players stop playing and look at the score.)
 */
  { "endyear", &game.end_year,
    SSET_META, SSET_TO_CLIENT,
    GAME_MIN_END_YEAR, GAME_MAX_END_YEAR, GAME_DEFAULT_END_YEAR,
    "Year the game ends", NULL },

  { "timeout", &game.timeout,
    SSET_META, SSET_TO_CLIENT,
    0, 999, GAME_DEFAULT_TIMEOUT,
    "Maximum seconds per turn", NULL },

  { "saveturns", &game.save_nturns,
    SSET_META, SSET_SERVER_ONLY,
    0, 200, 10,
    "Turns per auto-save", NULL },

  { "scorelog", &game.scorelog,
    SSET_META, SSET_SERVER_ONLY,
    GAME_MIN_SCORELOG, GAME_MAX_SCORELOG, GAME_DEFAULT_SCORELOG,
    "Whether to log player statistics", NULL }/*,

  { NULL, NULL,
    SSET_LAST, SSET_SERVER_ONLY,
    0, 0, 0,
    NULL, NULL } */
};


GtkWidget *setwin, *is_earth;


/*****************************************************************************
can the specified server setting (option) currently be changed?
*****************************************************************************/
int sset_is_changeable(int idx)
 {
  struct settings_s *op = &settings[idx];

  switch(op->sclass)
   {
    case SSET_MAP_SIZE:
    case SSET_MAP_GEN:
      /* Only change map options if we don't yet have a map: */
      return map_is_empty();	
    case SSET_MAP_ADD:
    case SSET_PLAYERS:
    case SSET_GAME_INIT:
    case SSET_RULES:
      /* Only change start params and most rules if we don't yet have a map,
       * or if we do have a map but its a scenario one.  Once a scenario is
       * actually started, game.scenario will be set to 0.
       */
      return (map_is_empty() || (game.scenario!=0));
    case SSET_RULES_FLEXIBLE:
    case SSET_META:
      /* These can always be changed: */
      return 1;
    default:
      return 0;
   }
 } /* sset_is_changeable() */


/*****************************************************************************
button callbacks for the settings dialog
*****************************************************************************/
void settings_ok(GtkWidget *widget, gpointer data)
 {
  gint i;

  for(i=0;i<NUM_SETTINGS;i++)
   {
    if(sset_is_changeable(i))
     {
      *(settings[i].value)=gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(settings[i].spinner));
     }
    settings[i].spinner=NULL;
   }

  i=map.is_earth;
  map.is_earth=GTK_TOGGLE_BUTTON(is_earth)->active;
  /* redraw the location of denmark if is_earth has changed */
  if(i!=map.is_earth)
    draw_denmark();

  gtk_widget_destroy(setwin);

 } /* settings_ok()*/


void settings_cancel(GtkWidget *widget, gpointer data)
 {
  gint i;

  for(i=0;i<NUM_SETTINGS;i++)
    settings[i].spinner=NULL;

  is_earth=NULL;
  gtk_widget_destroy(setwin);
 } /* settings_cancel*/


/*****************************************************************************
create the dialog to change map and game settings
*****************************************************************************/
void create_settings(void)
{
  GtkWidget *main_vbox;
  GtkWidget *table;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *spinner;
  GtkWidget *label;
  GtkAdjustment *adj;

  gint i;
  char s[32];

  setwin=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_modal(GTK_WINDOW(setwin), TRUE);
  gtk_signal_connect(GTK_OBJECT(setwin), "delete_event",
		     GTK_SIGNAL_FUNC(settings_cancel), NULL);
  gtk_window_set_title(GTK_WINDOW(setwin), "Map and Game Settings");

  main_vbox=gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 10);
  gtk_container_add(GTK_CONTAINER(setwin), main_vbox);

  table=gtk_table_new(TABLE_ROWS,(NUM_SETTINGS+TABLE_ROWS)/TABLE_ROWS+2,FALSE);
  gtk_box_pack_start(GTK_BOX(main_vbox), table, TRUE, TRUE, 0);

  for(i=0;i<NUM_SETTINGS;i++)
   {
    vbox=gtk_vbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
    gtk_table_attach(GTK_TABLE(table), vbox,
    		     i%TABLE_ROWS, i%TABLE_ROWS+1,
		     i/TABLE_ROWS, i/TABLE_ROWS+1,
		     GTK_FILL, GTK_FILL, 5, 5);

    sprintf(s,"%s (%i):", settings[i].name, settings[i].default_value);
    label=gtk_label_new(s);

    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

    if(sset_is_changeable(i))
     {
      adj=(GtkAdjustment *)gtk_adjustment_new(*settings[i].value,
					      (gfloat) settings[i].min_value,
					      (gfloat) settings[i].max_value,
					      1.0, 5.0, 0.0);
      spinner=gtk_spin_button_new(adj, 0, 0);
      gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinner), TRUE);
      gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spinner),
				      GTK_SHADOW_ETCHED_IN);
      gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinner), TRUE);
      settings[i].spinner=spinner;
     }
    else
     {
      settings[i].spinner=NULL;
      update_map_settings();
      sprintf(s," %i ", *(settings[i].value));
      spinner=gtk_label_new(s);
     }
    gtk_tooltips_set_tip(tooltips, spinner, settings[i].short_help, NULL);
    gtk_box_pack_start(GTK_BOX(vbox), spinner, FALSE, TRUE, 0);
   }

  vbox=gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_table_attach(GTK_TABLE(table), vbox,
    		   i%TABLE_ROWS, i%TABLE_ROWS+1,
		   i/TABLE_ROWS, i/TABLE_ROWS+1,
		   GTK_FILL, GTK_FILL, 5, 5);

  label=gtk_label_new(" ");
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

  is_earth=gtk_check_button_new_with_label("is_earth");
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(is_earth), map.is_earth);
  gtk_tooltips_set_tip(tooltips, is_earth,
		       "Should the denmark tiles be drawn?", NULL);
  gtk_box_pack_start(GTK_BOX(vbox), is_earth, FALSE, TRUE, 0);

  hbox=gtk_hseparator_new();
  gtk_table_attach(GTK_TABLE(table), hbox, 0, TABLE_ROWS,
		   i/TABLE_ROWS+1, i/TABLE_ROWS+2,
		   GTK_FILL, GTK_FILL, 0, 0);

  button=gtk_button_new_with_label("OK");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(settings_ok), NULL);
  gtk_table_attach(GTK_TABLE(table), button, 1, 2,
		   i/TABLE_ROWS+2, i/TABLE_ROWS+3,
		   GTK_FILL, GTK_FILL, 5, 5);

  button=gtk_button_new_with_label("Cancel");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(settings_cancel), NULL);
  gtk_table_attach(GTK_TABLE(table), button, TABLE_ROWS-2, TABLE_ROWS-1,
		   i/TABLE_ROWS+2, i/TABLE_ROWS+3,
		   GTK_FILL, GTK_FILL, 5, 5);

  gtk_widget_show_all(GTK_WIDGET(setwin));
  gtk_window_set_policy(GTK_WINDOW(setwin), FALSE, FALSE, FALSE);

 } /* create_settings() */


/****************************************************************************
button callbacks for clear specials dialog
*****************************************************************************/
void specials_ok(GtkWidget *widget, GtkWidget *dialog)
 {
  gtk_widget_destroy(widget);
  clear_rect_specials(0, 0, map.xsize, map.ysize);
  draw_everything(TRUE);
  refresh_screen();
 } /* specials_ok()*/


void specials_cancel(GtkWidget *widget, GtkWidget *dialog)
 {
  gtk_widget_destroy(widget);
 } /* specials_cancel*/


/*****************************************************************************
after confirmation, clear all specials from the map
*****************************************************************************/
void clear_specials(void)
 {
  GtkWidget *specialwin;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;

  specialwin=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(specialwin), "Clear all specials?");
  gtk_window_set_modal(GTK_WINDOW(specialwin), TRUE);
  gtk_signal_connect_object(GTK_OBJECT(specialwin), "delete_event",
		     GTK_SIGNAL_FUNC(specials_cancel), GTK_OBJECT(specialwin));

  vbox=gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_container_add(GTK_CONTAINER(specialwin), vbox);

  hbox=gtk_label_new("Are you sure that you want to\ndelete all specials from the map?\nThis can not be undone!");
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

  hbox=gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

  hbox=gtk_hbox_new(TRUE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

  button=gtk_button_new_with_label("Yes");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(specials_ok),
			    GTK_OBJECT(specialwin));
  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);

  button=gtk_button_new_with_label("No");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(specials_cancel),
			    GTK_OBJECT(specialwin));
  gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);

  gtk_widget_show_all(GTK_WIDGET(specialwin));
  gtk_window_set_policy(GTK_WINDOW(specialwin), FALSE, FALSE, FALSE);

 } /* clear_specials() */
