#include "tiles.h"
#include "draw.h"
#include "main.h"
#include "overview.h"

#include <mapview.h>
#include <map.h>
#include <game.h>
#include <ruleset.h>
/*
#include <city.h>
*/
/* from mapgen.c */
void init_workmap(void);

#include <stdlib.h>


gint TILE_WIDTH=30;
gint TILE_HEIGHT=30;
gboolean draw_diagonal_roads=TRUE;
gboolean have_pix_screen=TRUE;
GtkDrawingArea *da=NULL;

int terrain_to_tile_map[13]= {
  ARCTIC_TILES, DESERT_TILES, FOREST_TILES, GRASSLAND_TILES,
  HILLS_TILES, JUNGLE_TILES, MOUNTAINS_TILES, OCEAN_TILES,
  PLAINS_TILES, RIVER_TILES, SWAMP_TILES, TUNDRA_TILES
};

void draw_special(GtkWidget *widget, gint x, gint y, gint tileno);


/*****************************************************************************
produce terrain statistics from the current tile map
*****************************************************************************/
void update_map_settings(void)
 {
  gint x,y;
  int landmass=0,rivers=0,mountains=0,forests=0,swamps=0,deserts=0;

  for(x=0;x<map.xsize;x++)
    for(y=0;y<map.ysize;y++)
      switch(map_get_tile(x, y)->terrain)
       {
	case T_ARCTIC:
	case T_TUNDRA:
	case T_GRASSLAND:
	case T_PLAINS:
	  landmass++;
	break;
	case T_DESERT:
	  landmass++;
          deserts++;
	break;

	case T_JUNGLE:
	case T_FOREST:
	  landmass++;
          forests++;
	break;

	case T_HILLS:
	case T_MOUNTAINS:
	  landmass++;
	  mountains++;
	break;
	case T_SWAMP:
	  landmass++;
	  swamps++;
	break;
	case T_RIVER:
	  landmass++;
	  rivers++;
	break;
	case T_OCEAN:
	default: break; /* shouldn't happen */
       }

  map.landpercent=landmass*100/(map.xsize*map.ysize);
  /* intuitively, these should be 1/1000 chances based on landmass,
     but the map generators base them on the size of the map */
  landmass=map.xsize*map.ysize;
  map.mountains=mountains*1000/landmass;
  map.forestsize=forests*1000/landmass;
  map.swampsize=swamps*1000/landmass;
  map.deserts=deserts*1000/landmass;
  map.riverlength=rivers;
 } /* update_map_settings */


/*****************************************************************************
draws all tiles and units
*****************************************************************************/
void draw_everything(gboolean reuse_offscreen)
 {
  gint x, y, xd, yd;

  gtk_drawing_area_size(da, map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT);
  ov_set_size();

  if(!reuse_offscreen)
   {
    if(map.xsize*map.ysize*TILE_WIDTH*TILE_HEIGHT>OFF_SCREEN_MAX_SIZE)
      off_screen=FALSE;
    else
      off_screen=TRUE;
    /* if we already have an off-screen pixmap, dump it and make a new one */
    if(have_pix_screen && pix_screen)
      gdk_pixmap_unref(pix_screen);
    if(off_screen)
     {
      pix_screen=gdk_pixmap_new(win->window,
                                map.xsize*TILE_WIDTH, map.ysize*TILE_HEIGHT, -1);
      have_pix_screen=TRUE;
     }
    else
     {
      pix_screen=GTK_WIDGET(da)->window;
      have_pix_screen=FALSE;
     }
   }
  /* draw all terrain tiles on the off-screen pixmap */
  for(x=0;x<=map.xsize;x++)
   {
    for(y=0;y<=map.ysize;y++)
     {
      draw_tile(GTK_WIDGET(da),x,y, NULL);
     }
/*
      draw_tile(GTK_WIDGET(da),x,y, &city_to_draw_list);
     }
    draw_cities(GTK_WIDGET(da), &city_to_draw_list);
*/
   }

  /* draw the units as well */
  draw_units(GTK_WIDGET(da), 0, 0, map.xsize, map.ysize);

  /* How can scrolled window get scrollbars automatically? -> hack: resize! */
  gdk_window_get_size(win->window,&x,&y);
  get_deco_size(&xd, &yd);
  gdk_window_resize(win->window, min(x, map.xsize*TILE_WIDTH+xd),
                                 min(y-1, map.ysize*TILE_HEIGHT+yd));
 } /* draw_everything() */


/****************************************************************************
create a new game with default values
*****************************************************************************/
void create_new_map(gint x, gint y)
 {
  gint i;

  /* reset the map: delete the tiles*/
  if(map.tiles!=NULL)
    free(map.tiles);
  map.tiles=NULL;

  game_init();
  load_rulesets();

  game.scenario=3; /* default to type 3 senario (w/o start positions) */
  for(i=0;i<R_LAST;i++)
   {
    map.start_positions[i].x=0;
    map.start_positions[i].y=0;
   }
  map.xsize=x;
  map.ysize=y;
  init_workmap(); /* now create the map */

  /* draw the tiles: arctic in the polar region, ocean everywhere else */
  for(x=0;x<map.xsize;x++)
    for(y=0;y<map.ysize;y++)
      if((y==0) || (y==map.ysize-1))
	map_set_terrain(x, y, T_ARCTIC);
      else
	map_set_terrain(x,y,T_OCEAN);

 } /* create_new_map() */


/****************************************************************************
redraw the area of denmark on the maps
*****************************************************************************/
void draw_denmark(void)
 {
  gint x,y;
  for(x=34;x<=36;x++)
     for(y=map.ysize*.24;y<=map.ysize*.24+1;y++)
       draw_tile(GTK_WIDGET(da),x,y,NULL);
 } /* draw_denmark() */


/***************************************************************************
draw the terrain and specials of the tile at(x,y)
****************************************************************************/
void draw_tile(GtkWidget *widget, gint x, gint y, struct genlist *list)
 {
  int ttype, ttype_north, ttype_south, ttype_east, ttype_west;
  int ttype_north_east, ttype_south_east, ttype_south_west, ttype_north_west;
  int tspecial;
  int tspecial_north, tspecial_south, tspecial_east, tspecial_west;
  int tspecial_north_east, tspecial_south_east, tspecial_south_west, tspecial_north_west;
  int rail_card_tileno=0, rail_semi_tileno=0, road_card_tileno=0, road_semi_tileno=0;
  int rail_card_count=0, rail_semi_count=0, road_card_count=0, road_semi_count=0;
  int tileno;
  int adjacent;
  struct tile *ptile;
/*
  struct city *pcity;
*/
  gint den_y=map.ysize*.24;
  gint tmpX, tmpY;

  x=map_adjust_x(x);
  y=map_adjust_y(y);

  ptile=map_get_tile(x, y);

/*
  if(pcity=map_get_city(x, y))
    {
      if(list != NULL)
	draw_city(widget, pcity, list);
      return;
    }
*/

  ttype=map_get_terrain(x,y);
  ttype_north=map_get_terrain(x, y-1);
  ttype_south=map_get_terrain(x, y+1);
  ttype_west=map_get_terrain(x-1, y);
  ttype_east=map_get_terrain(x+1, y);
  ttype_north_east=map_get_terrain(x+1, y-1);
  ttype_south_east=map_get_terrain(x+1, y+1);
  ttype_south_west=map_get_terrain(x-1, y+1);
  ttype_north_west=map_get_terrain(x-1, y-1);

  tspecial=map_get_special(x, y);
  tileno=terrain_to_tile_map[ttype];

  switch(ttype)
   {
    case T_OCEAN:
      tileno+=(ttype_north!=T_OCEAN) ? 1 : 0;
      tileno+=(ttype_east!=T_OCEAN)  ? 2 : 0;
      tileno+=(ttype_south!=T_OCEAN) ? 4 : 0;
      tileno+=(ttype_west!=T_OCEAN)  ? 8 : 0;
    break;

    case T_RIVER:
      tileno=RIVER_TILES;
      tileno+=(ttype_north==T_RIVER || ttype_north==T_OCEAN) ? 1 : 0;
      tileno+=(ttype_east==T_RIVER  || ttype_east== T_OCEAN) ? 2 : 0;
      tileno+=(ttype_south==T_RIVER || ttype_south==T_OCEAN) ? 4 : 0;
      tileno+=(ttype_west==T_RIVER  || ttype_west== T_OCEAN) ? 8 : 0;
    break;

    case T_MOUNTAINS:
    case T_HILLS:
    case T_FOREST:
      if(ttype_west==ttype && ttype_east==ttype)
        tileno+=2;
      else if(ttype_west==ttype)
        tileno+=3;
      else if(ttype_east==ttype)
        tileno++;
    break;

    case T_GRASSLAND:
    case T_DESERT:
    case T_ARCTIC:
    case T_JUNGLE:
    case T_PLAINS:
    case T_SWAMP:
    case T_TUNDRA:
      tileno+=(ttype_north!=ttype) ? 1 : 0;
      tileno+=(ttype_east!=ttype)  ? 2 : 0;
      tileno+=(ttype_south!=ttype) ? 4 : 0;
      tileno+=(ttype_west!=ttype)  ? 8 : 0;
    break;
   } /* switch */

  /* if we are on Earth, place the denmark tiles at the appropriate location */
  if(map.is_earth && x>=34 && x<=36 && y>=den_y && y<=den_y+ 1)
    tileno=(y-den_y)*3+x-34+DENMARK_TILES;

  tmpX=tileno%20;
  tmpY=tileno/20;

  if(genlist_size((struct genlist *) &(ptile->units.list)) == 0)
   {
    gdk_draw_pixmap(pix_screen, widget->style->white_gc, pix_tiles,
		    tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		    x*TILE_WIDTH,y*TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT);
    ov_draw_tile(x, y, list);

    if(ttype==T_OCEAN)
     {
      tileno=CORNER_TILES-1;
      if((ttype_north==T_OCEAN && ttype_east==T_OCEAN &&
         ttype_north_east!=T_OCEAN))
        tileno+=1;
      if((ttype_east==T_OCEAN && ttype_south==T_OCEAN &&
         ttype_south_east!=T_OCEAN))
        tileno+=2;
      if((ttype_south==T_OCEAN && ttype_west==T_OCEAN &&
         ttype_south_west!=T_OCEAN))
        tileno+=4;
      if((ttype_north==T_OCEAN && ttype_west==T_OCEAN &&
         ttype_north_west!=T_OCEAN))
        tileno+=8;

      if(tileno!=CORNER_TILES-1)
        if(!(map.is_earth && x>=34 && x<=36 && y>=den_y && y<=den_y+ 1))
          draw_special(widget, x, y, tileno);

      if(ttype_north==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+0);
      if(ttype_west==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+1);
      if(ttype_south==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+2);
      if(ttype_east==T_RIVER)
        draw_special(widget, x, y, OUTLET_TILES+3);
     } /* if(T_OCEAN) */


    if(tspecial & S_IRRIGATION)
      draw_special(widget, x, y, IRRIGATION_TILE);

    if((tspecial & S_ROAD) ||(tspecial & S_RAILROAD))
     {
      tspecial_north=map_get_special(x, y-1);
      tspecial_east=map_get_special(x+1, y);
      tspecial_south=map_get_special(x, y+1);
      tspecial_west=map_get_special(x-1, y);

      tspecial_north_east=map_get_special(x+1, y-1);
      tspecial_south_east=map_get_special(x+1, y+1);
      tspecial_south_west=map_get_special(x-1, y+1);
      tspecial_north_west=map_get_special(x-1, y-1);

      rail_card_tileno+=(tspecial_north&S_RAILROAD) ? (rail_card_count++, 1) : 0;
      rail_card_tileno+=(tspecial_east&S_RAILROAD)  ? (rail_card_count++, 2) : 0;
      rail_card_tileno+=(tspecial_south&S_RAILROAD) ? (rail_card_count++, 4) : 0;
      rail_card_tileno+=(tspecial_west&S_RAILROAD)  ? (rail_card_count++, 8) : 0;

      road_card_tileno+=(tspecial_north&S_ROAD) ? (road_card_count++, 1) : 0;
      road_card_tileno+=(tspecial_east&S_ROAD)  ? (road_card_count++, 2) : 0;
      road_card_tileno+=(tspecial_south&S_ROAD) ? (road_card_count++, 4) : 0;
      road_card_tileno+=(tspecial_west&S_ROAD)  ? (road_card_count++, 8) : 0;

      rail_semi_tileno+=(tspecial_north_east&S_RAILROAD) ? (rail_semi_count++, 1) : 0;
      rail_semi_tileno+=(tspecial_south_east&S_RAILROAD) ? (rail_semi_count++, 2) : 0;
      rail_semi_tileno+=(tspecial_south_west&S_RAILROAD) ? (rail_semi_count++, 4) : 0;
      rail_semi_tileno+=(tspecial_north_west&S_RAILROAD) ? (rail_semi_count++, 8) : 0;

      road_semi_tileno+=(tspecial_north_east&S_ROAD) ? (road_semi_count++, 1) : 0;
      road_semi_tileno+=(tspecial_south_east&S_ROAD) ? (road_semi_count++, 2) : 0;
      road_semi_tileno+=(tspecial_south_west&S_ROAD) ? (road_semi_count++, 4) : 0;
      road_semi_tileno+=(tspecial_north_west&S_ROAD) ? (road_semi_count++, 8) : 0;

      if(tspecial & S_RAILROAD)
       {
        road_card_tileno&=~rail_card_tileno;
        road_semi_tileno&=~rail_semi_tileno;
       } else if(tspecial & S_ROAD)
       {
        rail_card_tileno&=~road_card_tileno;
        rail_semi_tileno&=~road_semi_tileno;
       }

      if(road_semi_count > road_card_count)
       {
        if(road_card_tileno!=0)
          draw_special(widget, x, y, ROAD_TILES+road_card_tileno);
        if(road_semi_tileno!=0 && draw_diagonal_roads)
          draw_special(widget, x, y, ROAD_TILES+16+road_semi_tileno);
       } else
       {
        if(road_semi_tileno!=0 && draw_diagonal_roads)
          draw_special(widget, x, y, ROAD_TILES+16+road_semi_tileno);
        if(road_card_tileno!=0)
          draw_special(widget, x, y, ROAD_TILES+road_card_tileno);
       }

      if(rail_semi_count > rail_card_count)
       {
        if(rail_card_tileno!=0)
          draw_special(widget, x, y, RAIL_TILES+rail_card_tileno);
        if(rail_semi_tileno!=0 && draw_diagonal_roads)
          draw_special(widget, x, y, RAIL_TILES+16+rail_semi_tileno);
       } else
       {
        if(rail_semi_tileno!=0 && draw_diagonal_roads)
          draw_special(widget, x, y, RAIL_TILES+16+rail_semi_tileno);
        if(rail_card_tileno!=0)
          draw_special(widget, x, y, RAIL_TILES+rail_card_tileno);
       }
     }

    if(tspecial & S_SPECIAL)
     {
      draw_special(widget, x, y, SPECIAL_TILES+ttype);
     }

    if(tspecial & S_MINE)
     {
      if(ttype==T_HILLS || ttype==T_MOUNTAINS)
	draw_special(widget, x, y, HILLMINE_TILE);
      else
	draw_special(widget, x, y, DESERTMINE_TILE);
     }

    if(tspecial & S_RAILROAD) {
      adjacent=rail_card_tileno;
      if(draw_diagonal_roads)
        adjacent |= rail_semi_tileno;
      if(!adjacent)
        draw_special(widget, x, y, RAIL_TILES);
    }
    else if(tspecial & S_ROAD) {
      adjacent=(rail_card_tileno | road_card_tileno);
      if(draw_diagonal_roads)
        adjacent|=(rail_semi_tileno | road_semi_tileno);
      if(!adjacent)
        draw_special(widget, x, y, ROAD_TILES);
    }

    if(tspecial & S_HUT)
      draw_special(widget, x, y, HUT_TILE);

    if(tspecial & S_FORTRESS)
      draw_special(widget, x, y, FORTRESS_TILE);

    if(tspecial & S_POLLUTION)
      draw_special(widget, x, y, POLLUTION_TILE);
   } /* if(genlist_size) */
 } /* draw_tile()*/


/***************************************************************************
draw the specified special on the tile at(x,y)
****************************************************************************/
void draw_special(GtkWidget *widget, gint x, gint y, gint tileno)
 {
  gint tmpX, tmpY;

  if(tileno<ROAD_TILES)
   {
    tmpX=tileno%20;
    tmpY=tileno/20;

    gdk_gc_set_clip_origin(gc_tiles,(x-tmpX)*TILE_WIDTH,(y-tmpY)*TILE_HEIGHT);
    gdk_draw_pixmap(pix_screen, gc_tiles, pix_tiles,
		     tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		     x*TILE_WIDTH, y*TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT);
   } else
   {
    tileno-=ROAD_TILES;
    tmpX=tileno%16;
    tmpY=tileno/16;

    gdk_gc_set_clip_origin(gc_roads,(x-tmpX)*TILE_WIDTH,(y-tmpY)*TILE_HEIGHT);
    gdk_draw_pixmap(pix_screen, gc_roads, pix_roads,
		     tmpX*TILE_WIDTH, tmpY*TILE_HEIGHT,
		     x*TILE_WIDTH, y*TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT);
   }
 } /* draw_special() */


/***************************************************************************
clear the specified rectangle of the map from specials
****************************************************************************/
void clear_rect_specials(gint x0, gint y0, gint x1, gint y1)
 {
  gint i,j;

  if(x0>x1)
   {
    i=x0;x0=x1;x1=i;
   }
  if(y0>y1)
   {
    i=y0;y0=y1;y1=i;
   }

  for(i=x0;i<x1;i++)
    for(j=y0;j<y1;j++)
      map_get_tile(i,j)->special=S_NONE;
 } /* clear_rect_specials() */


/***************************************************************************
clear the specified rectangle of the map from infrastructure
****************************************************************************/
void clear_rect_infrastructure(gint x0, gint y0, gint x1, gint y1)
 {
  gint i,j;

  if(x0>x1)
   {
    i=x0;x0=x1;x1=i;
   }
  if(y0>y1)
   {
    i=y0;y0=y1;y1=i;
   }

  for(i=x0;i<x1;i++)
    for(j=y0;j<y1;j++)
      map_get_tile(i,j)->special&=S_SPECIAL;
 } /* clear_rect_infrastructure() */


/*****************************************************************************
toggle the drawing of units on and off
*****************************************************************************/
void diagonal_roads(void)
 {
  draw_diagonal_roads=(!draw_diagonal_roads);
  draw_everything(TRUE);
  refresh_screen();
 } /* diagonal_roads() */


/*
void draw_cities(GtkWidget *widget, struct genlist *list)
 {
  struct genlist_iterator iterator;
  struct city *pcity;

  genlist_iterator_init(&iterator, list, 0);
  for(; ITERATOR_PTR(iterator); ITERATOR_NEXT(iterator))
   {
    pcity=(struct city *)ITERATOR_PTR(iterator);
    gdk_draw_string(pixmap, city_font, widget->style->white_gc,
		    pcity->x*TILE_WIDTH+15-
                    (gdk_string_width(city_font, pcity->name)/2),
		    pcity->y*TILE_HEIGHT+50, pcity->name);
   }
 }


void draw_city(GtkWidget *widget, struct city *pcity, struct genlist *list)
 {
  GdkFont *font;

  gdk_draw_pixmap(pixmap, widget->style->white_gc, pix_tiles,
		  game.players[pcity->owner].race*TILE_WIDTH, 14*TILE_HEIGHT,
		  pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		  TILE_WIDTH, TILE_HEIGHT);

  gdk_gc_set_clip_origin(gc_tiles,(pcity->x-((CITY_TILE)%20))*TILE_WIDTH,
 			 (pcity->y -((CITY_TILE)/20))*TILE_HEIGHT);
  gdk_draw_pixmap(pixmap, gc_tiles, pix_tiles, (((city_got_citywalls(pcity)) ?
		  (CITY_WALLS_TILE):(CITY_TILE))%20)*TILE_WIDTH,
  		  ((CITY_TILE)/20)*TILE_HEIGHT,
		  pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		  TILE_WIDTH, TILE_HEIGHT);

  gdk_gc_set_clip_origin(gc_tiles,(pcity->x-(pcity->size%10))*TILE_WIDTH,
 			 (pcity->y-((NUMBER_TILES)/20))*TILE_HEIGHT);
  gdk_draw_pixmap(pixmap, gc_tiles, pix_tiles,
  		  (pcity->size%10)*TILE_WIDTH,((NUMBER_TILES)/20)*TILE_HEIGHT,
		  pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		  TILE_WIDTH, TILE_HEIGHT);

  if(pcity->size > 9)
   {
    gdk_gc_set_clip_origin(gc_tiles, (pcity->x-((NUMBER_MSD_TILES)%20)-
			   (pcity->size/10))*TILE_WIDTH,
			   (pcity->y-((NUMBER_MSD_TILES)/20))*TILE_HEIGHT);
    gdk_draw_pixmap(pixmap, gc_tiles, pix_tiles,
		    (((NUMBER_MSD_TILES)%20)+(pcity->size/10))*TILE_WIDTH,
		    ((NUMBER_MSD_TILES)/20)*TILE_HEIGHT,
		    pcity->x*TILE_WIDTH, pcity->y*TILE_HEIGHT,
		    TILE_WIDTH, TILE_HEIGHT);
   }

  genlist_insert(list, pcity, 0);

 }
*/
