#include "toolfunc.h"
#include "undo.h"
#include "draw.h"
#include "tiles.h"
#include "civ.h"

#include <map.h>
#include <game.h>

#include <stdlib.h>
#include <stdio.h>

gint country=0;
gint current_color=T_ARCTIC;

gboolean can_draw=TRUE;

gint xstart=-1,ystart=-1;
gint xold=-1,yold=-1;

GdkGC *gc;

/* the clipboard structure */
typedef struct _clipboad
 {
  gint width, height;
  struct tile *tiles;
  struct map_position start_positions[R_LAST];
 } Clipboard;

/* the clipboard */
Clipboard clip;

/*****************************************************************************
clear the selection coordinates, then refresh the screen
*****************************************************************************/
void clear_sel(void)
 {
  xold=xstart=-1;
  yold=ystart=-1;
  refresh_screen();
 } /* clear_sel() */


/*****************************************************************************
change the color, i.e. terrain, of the tile at (x,y)
*****************************************************************************/
void start_tile(GtkWidget *widget, gint x, gint y)
 {
  struct tile *ptile;
  gint i, j;

  x/=TILE_WIDTH;
  y/=TILE_HEIGHT;

  if((x>=map.xsize) || (y>=map.ysize))
    return;

  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x==x) &&
       (map.start_positions[i].y==y) &&
       (current_color!=T_GRASSLAND) &&
       (current_color!=T_PLAINS) &&
       (current_color!=T_RIVER))
      return;

  prime_undo(x, y, x, y);

  ptile=map_get_tile(x, y);
  ptile->terrain=current_color;

  for(i=-1; i<=1; i++)
   {
    for(j=-1; j<=1; j++)
      draw_tile(widget, x+i, y+j, NULL);
   }
  refresh_screen();
 } /* start_tile() */


/*****************************************************************************
update map information after tile change
*****************************************************************************/
void end_tile(GtkWidget *widget, gint x, gint y)
 {
  update_map_settings();
 } /* end_tile() */


/*****************************************************************************
start drawing of rectangle: save start tile, draw rubber band rectangle
*****************************************************************************/
void start_rect(GtkWidget *widget, gint x, gint y)
 {
  x+=TILE_WIDTH/2;
  y+=TILE_HEIGHT/2;
  if((x/TILE_WIDTH>map.xsize) || (y/TILE_HEIGHT>map.ysize))
    return;

  if((xstart==-1) && (ystart==-1))
    {
     xstart=(x/TILE_WIDTH)*TILE_WIDTH;
     ystart=(y/TILE_HEIGHT)*TILE_HEIGHT;
     xold=(x/TILE_WIDTH)*TILE_WIDTH;
     yold=(y/TILE_HEIGHT)*TILE_HEIGHT;
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);
    } else
    {
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);
     xold=(x/TILE_WIDTH)*TILE_WIDTH;
     yold=(y/TILE_HEIGHT)*TILE_HEIGHT;
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);
    }
 } /* start_rect() */


/*****************************************************************************
undraw the rectangle, update the map stats, clear the start tile coords
*****************************************************************************/
void end_rect(GtkWidget *widget, gint x, gint y)
 {
  struct tile *ptile;
  gint i;

  x+=TILE_WIDTH/2;
  y+=TILE_HEIGHT/2;
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);

  if((x/TILE_WIDTH>map.xsize) || (y/TILE_HEIGHT>map.ysize))
   {
    xold=xstart=-1;
    yold=ystart=-1;
    return;
   }

  x=(x/TILE_WIDTH)*TILE_WIDTH;
  y=(y/TILE_HEIGHT)*TILE_HEIGHT;
  if(x<xstart)
   {
    xold=x;x=xstart;xstart=xold;
   }
  if(y<ystart)
   {
    yold=y;y=ystart;ystart=yold;
   }

  prime_undo(xstart/TILE_WIDTH, ystart/TILE_HEIGHT,
	     x/TILE_WIDTH-1, y/TILE_HEIGHT-1);
  for(xold=xstart/TILE_WIDTH; xold<x/TILE_WIDTH; xold++)
    for(yold=ystart/TILE_HEIGHT; yold<y/TILE_HEIGHT; yold++)
     {
      can_draw=TRUE;
      for(i=0;i<R_LAST;i++)
        if((map.start_positions[i].x==xold) &&
           (map.start_positions[i].y==yold) &&
           (current_color!=T_GRASSLAND) &&
           (current_color!=T_PLAINS) &&
           (current_color!=T_RIVER))
          can_draw=FALSE;

      if(can_draw)
       {
        ptile=map_get_tile(xold, yold);
        ptile->terrain=current_color;
       }
     }

  for(xold=xstart/TILE_WIDTH-1;xold<=x/TILE_WIDTH+1;xold++)
    for(yold=ystart/TILE_WIDTH-1;yold<=y/TILE_WIDTH+1;yold++)
       draw_tile(widget, xold, yold, NULL);

  clear_sel();
  update_map_settings();
 } /* end_rect() */


/*****************************************************************************
start drawing of line: save start tile, draw rubber band line
*****************************************************************************/
void start_line(GtkWidget *widget, gint x, gint y)
 {
  if((x/TILE_WIDTH>=map.xsize) || (y/TILE_HEIGHT>=map.ysize))
    return;

  if((xstart==-1) && (ystart==-1))
    {
     xstart=(x/TILE_WIDTH)*TILE_WIDTH+TILE_WIDTH/2;
     ystart=(y/TILE_HEIGHT)*TILE_HEIGHT+TILE_HEIGHT/2;
     xold=(x/TILE_WIDTH)*TILE_WIDTH+TILE_WIDTH/2;
     yold=(y/TILE_HEIGHT)*TILE_HEIGHT+TILE_HEIGHT/2;
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xold, yold);
    } else
    {
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xold, yold);
     xold=(x/TILE_WIDTH)*TILE_WIDTH+TILE_WIDTH/2;
     yold=(y/TILE_HEIGHT)*TILE_HEIGHT+TILE_HEIGHT/2;
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xold, yold);
    }
 } /* start_line() */


/*****************************************************************************
draw the line, update the map stats, clear the start tile coords
*****************************************************************************/
void end_line(GtkWidget *widget, gint x, gint y)
 {
  struct tile *ptile;
  gint dx,dy,xt,yt,i;

  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xold, yold);

  if((x/TILE_WIDTH>=map.xsize) || (y/TILE_HEIGHT>=map.ysize))
   {
    xold=xstart=-1;
    yold=ystart=-1;
    return;
   }

  x=(x/TILE_WIDTH)*TILE_WIDTH+TILE_WIDTH/2;
  y=(y/TILE_HEIGHT)*TILE_HEIGHT+TILE_WIDTH/2;
  dx=x-xstart;
  dy=y-ystart;

  if((dx==0) && (dy==0))
   {
    clear_sel();
    return;
   }

  if(abs(dx) > abs(dy))
   {
    if(x<xstart)
     {
      xold=x;x=xstart;xstart=xold;
     }
    if(dx<0)
     {
      yold=y;y=ystart;ystart=yold;
     }
    prime_undo(xstart/TILE_WIDTH, ystart/TILE_HEIGHT,
	       x/TILE_WIDTH, y/TILE_WIDTH);
    for(xold=xstart; xold<=x; xold+=TILE_WIDTH)
     {
      yold=ystart+(dy*(xold-xstart))/dx;
      xt=xold/TILE_WIDTH;
      yt=yold/TILE_HEIGHT;
      can_draw=TRUE;
      for(i=0;i<R_LAST;i++)
        if((map.start_positions[i].x==xt) &&
           (map.start_positions[i].y==yt) &&
           (current_color!=T_GRASSLAND) &&
           (current_color!=T_PLAINS) &&
           (current_color!=T_RIVER))
          can_draw=FALSE;

      if(can_draw)
       {
        ptile=map_get_tile(xt, yt);
        ptile->terrain=current_color;
       }
     }
    for(xold=xstart-TILE_WIDTH; xold<=x+TILE_WIDTH; xold+=TILE_WIDTH)
     {
      yold=ystart+(dy*(xold-xstart))/dx;
      xt=xold/TILE_WIDTH;
      yt=yold/TILE_HEIGHT;
      draw_tile(widget, xt, yt-2, NULL);
      draw_tile(widget, xt, yt-1, NULL);
      draw_tile(widget, xt, yt, NULL);
      draw_tile(widget, xt, yt+1, NULL);
      draw_tile(widget, xt, yt+2, NULL);
     }
   }
  else /* if(abs(dx) > abs(dy)) */
   {
    if(y<ystart)
     {
      yold=y;y=ystart;ystart=yold;
     }
    if(dy<0)
     {
      xold=x;x=xstart;xstart=xold;
     }
    prime_undo(xstart/TILE_WIDTH, ystart/TILE_HEIGHT,
	       x/TILE_WIDTH, y/TILE_WIDTH);
    for(yold=ystart; yold<=y; yold+=TILE_HEIGHT)
     {
      xold=xstart+(dx*(yold-ystart))/dy;
      xt=xold/TILE_WIDTH;
      yt=yold/TILE_HEIGHT;
      can_draw=TRUE;
      for(i=0;i<R_LAST;i++)
        if((map.start_positions[i].x==xt) &&
           (map.start_positions[i].y==yt) &&
           (current_color!=T_GRASSLAND) &&
           (current_color!=T_PLAINS) &&
           (current_color!=T_RIVER))
          can_draw=FALSE;

      if(can_draw)
       {
        ptile=map_get_tile(xt, yt);
        ptile->terrain=current_color;
       }
     }
    for(yold=ystart-TILE_HEIGHT; yold<=y+TILE_HEIGHT; yold+=TILE_HEIGHT)
     {
      xold=xstart+(dx*(yold-ystart))/dy;
      xt=xold/TILE_WIDTH;
      yt=yold/TILE_HEIGHT;
      draw_tile(widget, xt-2, yt, NULL);
      draw_tile(widget, xt-1, yt, NULL);
      draw_tile(widget, xt, yt, NULL);
      draw_tile(widget, xt+1, yt, NULL);
      draw_tile(widget, xt+2, yt, NULL);
     }
   }

  clear_sel();
  update_map_settings();
 } /* end_line() */


/*****************************************************************************
change the start position of the current race to (x,y)
*****************************************************************************/
void put_unit(GtkWidget *widget, gint x, gint y)
 {
  gint i, j;
  gint xo, yo;

  x/=TILE_WIDTH;
  y/=TILE_HEIGHT;

  if((x>=map.xsize) || (y>=map.ysize))
    return;

  /* don't place start positions on unfriendly terrain */
  if (map_get_terrain(x, y)!=T_PLAINS &&
      map_get_terrain(x, y)!=T_GRASSLAND &&
      map_get_terrain(x, y)!=T_RIVER)
    return;


  /* don't stack start positions */
  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x==x) && (map.start_positions[i].y==y))
      return;

  prime_undo(x, y, x, y);
  /* save the old start position */
  xo=map.start_positions[country].x;
  yo=map.start_positions[country].y;

  /* enter the new start position into the map structure */
  map.start_positions[country].x=x;
  map.start_positions[country].y=y;

  /* delete the settler unit picture from the old start position */
  for(i=-1; i<=1; i++)
    for(j=-1; j<=1; j++)
      draw_tile(widget, xo+i, yo+j, NULL);

  /* with the possibly new start position, the admissible scenario types may
     have changed. Check this: */
  if(game.scenario==3)
    set_scenario();
  draw_units(widget, 0, 0, map.xsize*TILE_WIDTH,map.ysize*TILE_HEIGHT);
  refresh_screen();
 } /* put_unit() */


/*****************************************************************************
toggle the special flag on the tile
*****************************************************************************/
void toggle_special(GtkWidget *widget, gint x, gint y)
 {
  struct tile *ptile;

  x/=TILE_WIDTH;
  y/=TILE_HEIGHT;

  if((x>=map.xsize) || (y>=map.ysize))
    return;

  prime_undo(x, y, x, y);
  ptile=map_get_tile(x, y);
  if(ptile->special & S_SPECIAL)
    ptile->special&=~S_SPECIAL;
  else
    ptile->special|=S_SPECIAL;
  draw_tile(widget, x, y, NULL);
  refresh_screen();
 } /* toggle_special() */


/*****************************************************************************
cut: copy the rectangle of tiles into the clipboard, then overwrite with ocean
*****************************************************************************/
void end_cut(GtkWidget *widget, gint x, gint y)
 {
  gint xo, xs, yo, ys;
  gint i;

  xs=xstart;
  ys=ystart;

  end_copy(widget, x, y);

  x+=TILE_WIDTH/2;
  y+=TILE_HEIGHT/2;

  if((x/TILE_WIDTH>map.xsize) || (y/TILE_HEIGHT>map.ysize))
    return;

  x=(x/TILE_WIDTH)*TILE_WIDTH;
  y=(y/TILE_HEIGHT)*TILE_HEIGHT;
  if(x<xs)
   {
    xo=x;x=xs;xs=xo;
   }
  if(y<ys)
   {
    yo=y;y=ys;ys=yo;
   }

  xs/=TILE_WIDTH;
  x/=TILE_WIDTH;
  ys/=TILE_HEIGHT;
  y/=TILE_HEIGHT;

  if((x==xs) || (y==ys))
    return;

  prime_undo(xs, ys, x, y);
  /* copy relative start positions from the cut area into the clipboard,
     set them to 0 in the map, else set them to -1 in the clipboard */
  for(i=0;i<R_LAST;i++)
    if((map.start_positions[i].x>=xs) && (map.start_positions[i].x<x) &&
       (map.start_positions[i].y>=ys) && (map.start_positions[i].y<y))
      {
       clip.start_positions[i].x=map.start_positions[i].x-xs;
       clip.start_positions[i].y=map.start_positions[i].y-ys;
       map.start_positions[i].x=0;
       map.start_positions[i].y=0;
      }
     else
      {
       clip.start_positions[i].x=-1;
       clip.start_positions[i].y=-1;
      }

  for(xo=xs; xo<x; xo++)
    for(yo=ys; yo<y; yo++)
     {
      map_get_tile(xo, yo)->terrain=T_OCEAN;
      map_get_tile(xo, yo)->special=S_NONE;
     }

  for(xo=xs-1; xo<=x; xo++)
    for(yo=ys-1; yo<=y; yo++)
      draw_tile(widget, xo, yo, NULL);

  refresh_screen();
  update_map_settings();
 } /* end_cut() */


/*****************************************************************************
copy: copy the rectangle of tiles into the clipboard;
*****************************************************************************/
void end_copy(GtkWidget *widget, gint x, gint y)
 {
  gint i;

  x+=TILE_WIDTH/2;
  y+=TILE_HEIGHT/2;
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);

  if((x/TILE_WIDTH>map.xsize) || (y/TILE_HEIGHT>map.ysize))
   {
    xold=xstart=-1;
    yold=ystart=-1;
    return;
   }

  x=(x/TILE_WIDTH)*TILE_WIDTH;
  y=(y/TILE_HEIGHT)*TILE_HEIGHT;
  if(x<xstart)
   {
    xold=x;x=xstart;xstart=xold;
   }
  if(y<ystart)
   {
    yold=y;y=ystart;ystart=yold;
   }

  xstart/=TILE_WIDTH;
  x/=TILE_WIDTH;
  ystart/=TILE_HEIGHT;
  y/=TILE_HEIGHT;

  if((x-xstart==0) || (y-ystart==0))
    return;

  clip.width=x-xstart;
  clip.height=y-ystart;

  if(clip.tiles!=NULL)
    free(clip.tiles);
  clip.tiles=NULL;
  if(!(clip.tiles=(struct tile*)malloc((clip.width)*(clip.height)*
     sizeof(struct tile))))
   {
    fprintf(stderr, "malloc failed in toolfunc\n");
    exit(1);
   }

  /* copy the rectangle into the clipboard */
  i=0;
  for(xold=xstart; xold<x; xold++)
    for(yold=ystart; yold<y; yold++)
      memcpy(clip.tiles+(i++), map_get_tile(xold,yold), sizeof(struct tile));

  for(i=0;i<R_LAST;i++)
   {
    clip.start_positions[i].x=-1;
    clip.start_positions[i].y=-1;
   }

  xold=xstart=-1;
  yold=ystart=-1;
 } /* end_copy() */


/*****************************************************************************
copy: copy the rectangle of tiles into the clipboard;
*****************************************************************************/
void start_paste(GtkWidget *widget, gint x, gint y)
 {
  x+=TILE_WIDTH/2;
  y+=TILE_HEIGHT/2;

  if((x/TILE_WIDTH>map.xsize) || (y/TILE_HEIGHT>map.ysize))
    return;

  if((xstart!=-1) && (ystart!=-1))
    {
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
     gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);
    }
  xstart=(x/TILE_WIDTH)*TILE_WIDTH;
  ystart=(y/TILE_HEIGHT)*TILE_HEIGHT;
  xold=xstart+clip.width*TILE_WIDTH;
  yold=ystart+clip.height*TILE_HEIGHT;
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);
 } /* start_paste() */


/*****************************************************************************
paste: copy the rectangle of tiles from the clipboard;
*****************************************************************************/
void end_paste(GtkWidget *widget, gint x, gint y)
 {
  gint i, j, terrain;

  x+=TILE_WIDTH/2;
  y+=TILE_HEIGHT/2;
  if((x/TILE_WIDTH>map.xsize) || (y/TILE_HEIGHT>map.ysize))
   {
    xold=xstart=-1;
    yold=ystart=-1;
    return;
   }

  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, ystart, xstart, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xstart, yold, xold, yold);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, yold, xold, ystart);
  gdk_draw_line(GTK_WIDGET(da)->window, gc, xold, ystart, xstart, ystart);

  xstart/=TILE_WIDTH;
  xold/=TILE_WIDTH;
  ystart/=TILE_HEIGHT;
  yold/=TILE_HEIGHT;

  prime_undo(xstart, ystart, xold-1, min(yold, map.ysize)-1);
  /* copy the tiles into the new place on the map */
  i=0;
  for(x=xstart; x<xold; x++)
    for(y=ystart; y<yold; y++)
      if(y>=map.ysize)
        i++;
      else
        memcpy(map_get_tile(map_adjust_x(x),y), clip.tiles+(i++),
	       sizeof(struct tile));

  /* copy the start positions */
  for(i=0;i<R_LAST;i++)
   {
    if((clip.start_positions[i].x!=-1) && (clip.start_positions[i].y!=-1))
     {
      x=map.start_positions[i].x;
      y=map.start_positions[i].y;
      map.start_positions[i].x=clip.start_positions[i].x+xstart;
      map.start_positions[i].y=clip.start_positions[i].y+ystart;
      draw_tile(widget, x, y, NULL);
      /* delete start positions when there's a clash */
      for(j=0;j<R_LAST;j++)
        if((map.start_positions[j].x==map.start_positions[i].x) &&
	   (map.start_positions[j].y==map.start_positions[i].y) && (i!=j))
         {
	  map.start_positions[j].x=0;
	  map.start_positions[j].y=0;
         }
     }
    else
     {
      /* delete start positions on illegal terrain */
      terrain=map_get_terrain(map.start_positions[i].x,
			      map.start_positions[i].y);
      if((terrain!=T_GRASSLAND) && (terrain!=T_PLAINS) &&
         (terrain!=T_RIVER))
       {
        map.start_positions[i].x=0;
        map.start_positions[i].y=0;
       }
     }
   }
  draw_tile(widget, 0, 0, NULL); /* show the units at (0,0) in the overview */

  /* draw the new area */
  for(x=xstart-1;x<=xold+1;x++)
    for(y=ystart-1;y<=yold+1;y++)
       draw_tile(widget, x, y, NULL);

  draw_units(widget, 0, 0, map.xsize, map.ysize);

  clear_sel();
  update_map_settings();
 } /* end_paste() */


/*****************************************************************************
initialize the clipboard
*****************************************************************************/
void init_clipboard(void)
 {
  gint i;

  clip.width=clip.height=0;
  if(clip.tiles!=NULL)
    free(clip.tiles);
  clip.tiles=NULL;
  for(i=0;i<R_LAST;i++)
   {
    clip.start_positions[i].x=-1;
    clip.start_positions[i].y=-1;
   }
 } /* init_clipboard() */
