#include "tools.h"
#include "undo.h"
#include "draw.h"
#include "tiles.h"
#include "main.h"
#include "menu.h"

#include "map.h"

#include <stdio.h>

#define TABLE_WIDTH 3
#define TOOL_WIDTH 5

/* Tool pixmaps */
#include "data/crayon.xpm"
#include "data/ligne.xpm"
#include "data/rectangle.xpm"
#include "data/unit.xpm"
#include "data/special.xpm"
#include "data/cut.xpm"
#include "data/copy.xpm"
#include "data/paste.xpm"
#include "data/undo.xpm"


gboolean can_change_race=FALSE;
gboolean tooltips_enabled=TRUE;
gboolean tools_shown=TRUE;

GtkWidget *toolwin;
char labl[25];

gint destroyWindow(GtkWidget *widget, GdkEvent *ev, gpointer data);

void create_palette(GtkWidget *parent);
void create_colors(GtkWidget *widget);
void set_color(GtkWidget *widget, gpointer color);

void create_tools(GtkWidget *win, GtkWidget *parent);
void tool_toggled(GtkWidget *widget, gpointer tool);


typedef struct _couleur
 {
  char *name;
  gint parametre;
  GtkWidget *button;
  gint id;
 } Couleur;

Couleur couleurs[NB_COLORS] =
 {
  {"Artic", T_ARCTIC, NULL, 0},
  {"Desert", T_DESERT, NULL, 0},
  {"Forest", T_FOREST, NULL, 0},
  {"Grassland", T_GRASSLAND, NULL, 0},
  {"Hills", T_HILLS, NULL, 0},
  {"Jungle", T_JUNGLE, NULL, 0},
  {"Mountains", T_MOUNTAINS, NULL, 0},
  {"Ocean", T_OCEAN, NULL, 0},
  {"Plains", T_PLAINS, NULL, 0},
  {"River", T_RIVER, NULL, 0},
  {"Swamp", T_SWAMP, NULL, 0},
  {"Tundra", T_TUNDRA, NULL, 0},
 };

Tool tools[TOOLS_NUMBER]=
 {
  {"Point", crayon_xpm, (GdkPixmap *) NULL, NULL,
   "Draw a single map tile",
   start_tile, end_tile, NULL, 0},
  {"Line", ligne_xpm, (GdkPixmap *) NULL, NULL,
   "Draw a line of map tiles",
   start_line, end_line, NULL, 0},
  {"Rectangle", rectangle_xpm, (GdkPixmap *) NULL, NULL,
   "Draw a rectangle of map tiles",
   start_rect, end_rect, NULL, 0},
  {"Special", special_xpm, (GdkPixmap *) NULL, NULL,
   "Toggle the \"special\" flag on a terrain tile, which is responsible for things like gold in mountains and horses on plains.",
   toggle_special, (ToolFunction) NULL, NULL, 0},
  {"Unit", unit_xpm, (GdkPixmap *) NULL, NULL,
   "Place the starting position for the currently selected race. ALL races must have a start position before those can be saved.",
   put_unit, (ToolFunction) NULL, NULL, 0},
  {"Cut", cut_xpm, (GdkPixmap *) NULL, NULL,
   "Cut out a rectangular area into the clipboard",
   start_rect, end_cut, NULL, 0},
  {"Copy", copy_xpm, (GdkPixmap *) NULL, NULL,
   "Copy a rectangular area into the clipboard",
   start_rect, end_copy, NULL, 0},
  {"Paste", paste_xpm, (GdkPixmap *) NULL, NULL,
   "Paste a rectangular area from the clipboard into the map",
   start_paste, end_paste, NULL, 0},
  {"Undo", undo_xpm, (GdkPixmap *) NULL, NULL,
   "Undo the last operation",
   (ToolFunction) NULL, (ToolFunction) NULL, NULL, 0}
 };


/****************************************************************************
change the active country
*****************************************************************************/
void change_race(void)
 {
  if(can_change_race)
   {
    country=(++country) % R_LAST;
    refresh_racelabel();
   }
 } /* change_race() */


/****************************************************************************
set the drawing color (=terrain) to the currently selected one
*****************************************************************************/
void set_color(GtkWidget *widget, gpointer color)
 {
  gint i;

  current_color=couleurs[GPOINTER_TO_INT(color)].parametre;

  /* press the current button and unpress all others */
  for(i=0; i<NB_COLORS; i++)
   {
    gtk_signal_handler_block(GTK_OBJECT(couleurs[i].button), couleurs[i].id);
    if(i!=GPOINTER_TO_INT(color))
     {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(couleurs[i].button), FALSE);
     } else {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(couleurs[i].button), TRUE);
     }
    gtk_signal_handler_unblock(GTK_OBJECT(couleurs[i].button),couleurs[i].id);
   }
  sprintf(labl,"%s ",couleurs[GPOINTER_TO_INT(color)].name);
  set_terrain_label(labl);
 } /* set_color() */


/****************************************************************************
create the palette and fill it with the terrain "colors"
*****************************************************************************/
void create_palette(GtkWidget *parent)
 {
  GtkWidget *table;
  gint i;

  table=gtk_table_new(TABLE_WIDTH, (NB_COLORS+TABLE_WIDTH-1)/TABLE_WIDTH-1, TRUE);
  gtk_box_pack_start(GTK_BOX(parent), table, TRUE, FALSE, 0);

  for(i=0; i<NB_COLORS; i++)
   {
    couleurs[i].button=gtk_toggle_button_new_with_label(couleurs[i].name);
    gtk_table_attach(GTK_TABLE(table), couleurs[i].button,
		     i%TABLE_WIDTH, i%TABLE_WIDTH+1,
		     i/TABLE_WIDTH, i/TABLE_WIDTH+1, GTK_FILL, GTK_FILL, 0, 0);
    couleurs[i].id=gtk_signal_connect(GTK_OBJECT(couleurs[i].button),
				      "toggled", GTK_SIGNAL_FUNC(set_color),
                                      GINT_TO_POINTER(i));
    gtk_widget_show(couleurs[i].button);
   }

  current_color=couleurs[0].parametre;
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(couleurs[0].button), TRUE);
  gtk_widget_show(table);
 } /* create_palette */


/***************************************************************************
fill the tools[]-array with the missing values: pixmaps, buttons, ids
***************************************************************************/
void create_tools(GtkWidget *win, GtkWidget *parent)
 {
  gint i;
  GtkWidget *pixmap;
  GtkWidget *table;

  table=gtk_table_new(TOOL_WIDTH, (TOOLS_NUMBER+TOOL_WIDTH-1)/TOOL_WIDTH, TRUE);
  gtk_box_pack_start(GTK_BOX(parent), table, TRUE, FALSE, 0);

  /* create the button label pixmaps from the xpm's */
  for(i=0; i<TOOLS_NUMBER; i++)
   {
    if(tools[i].pixmap_data != NULL)
     {
      tools[i].pixmap=gdk_pixmap_create_from_xpm_d(
        win->window, &tools[i].mask, &white, tools[i].pixmap_data);
     }
    else
     {
      tools[i].pixmap=NULL;
     }
   }

  for(i=0; i<TOOLS_NUMBER; i++)
   {
    tools[i].button=gtk_toggle_button_new();
    if(tools[i].pixmap != NULL)
     {
      pixmap=gtk_pixmap_new(tools[i].pixmap, tools[i].mask);
      gtk_container_add(GTK_CONTAINER(tools[i].button), pixmap);
      gtk_widget_show(pixmap);
     }
    gtk_table_attach(GTK_TABLE(table), tools[i].button,
		     i%TOOL_WIDTH, i%TOOL_WIDTH+1,
		     i/TOOL_WIDTH, i/TOOL_WIDTH+1, GTK_FILL, GTK_FILL, 0, 0);
    tools[i].id=gtk_signal_connect(GTK_OBJECT(tools[i].button),
				   "toggled", GTK_SIGNAL_FUNC(tool_toggled),
				   GINT_TO_POINTER(i));
    gtk_tooltips_set_tip(tooltips, tools[i].button, tools[i].tiptext, NULL);
    gtk_widget_show(tools[i].button);
   }

  current_tool=&tools[0];
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tools[0].button), TRUE);
  gtk_widget_show(table);
 } /* create_tools() */


/***************************************************************************
create the Toolbar window
***************************************************************************/
void create_tools_palette()
 {
  GtkWidget *vbox;
  GtkWidget *hbox;

  gc=gdk_gc_new(GTK_WIDGET(da)->window);
  gdk_gc_set_function(gc, GDK_INVERT);

  toolwin=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(toolwin), "Toolbar");
  gtk_window_set_transient_for(GTK_WINDOW(toolwin),GTK_WINDOW(win));
  gtk_widget_set_usize(GTK_WIDGET(toolwin),200,180);
  gtk_container_set_border_width(GTK_CONTAINER(toolwin), 5);
  gtk_window_set_policy(GTK_WINDOW(toolwin), FALSE, FALSE, FALSE);
  gtk_signal_connect(GTK_OBJECT(toolwin), "delete_event",
		     GTK_SIGNAL_FUNC(destroyWindow), toolwin);

  vbox=gtk_vbox_new(FALSE, 5);
  gtk_container_add(GTK_CONTAINER(toolwin), vbox);
  gtk_widget_show(vbox);
  gtk_widget_show(toolwin);

  tooltips=gtk_tooltips_new();
  init_undo();
  init_clipboard();

  /* tool button creation */
  hbox=gtk_hbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
  create_tools(toolwin, hbox);
  gtk_widget_show(hbox);

  /* palette creation */
  hbox=gtk_hbox_new(TRUE, 0);
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
  create_palette(hbox);
  gtk_widget_show(hbox);

 } /* create_tools_palette() */


/*****************************************************************************
handle the toggle buttons' toggle events
*****************************************************************************/
void tool_toggled(GtkWidget *widget, gpointer tool)
 {
  gint i,j;

  j=GPOINTER_TO_INT(tool);
  if(j==8) /* undo selected */
   {
    gtk_signal_handler_block(GTK_OBJECT(tools[j].button), tools[j].id);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tools[j].button), FALSE);
    gtk_signal_handler_unblock(GTK_OBJECT(tools[j].button), tools[j].id);
    do_undo();
    return;
   }

  current_tool=&tools[j];

  /* press the current button and unpress all others*/
  for(i=0; i<TOOLS_NUMBER; i++)
   {
    gtk_signal_handler_block(GTK_OBJECT(tools[i].button), tools[i].id);
    if(i!=j)
     {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tools[i].button), FALSE);
     } else {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tools[i].button), TRUE);
     }
     gtk_signal_handler_unblock(GTK_OBJECT(tools[i].button), tools[i].id);
   }

  /* ugly code here: j>=3 means no tile drawing function selected */
  for(i=0;i<NB_COLORS;i++)
   {
    if(couleurs[i].button!=NULL)
      if(j>=3)
        gtk_widget_set_sensitive(couleurs[i].button,FALSE);
      else
        gtk_widget_set_sensitive(couleurs[i].button,TRUE);
   }

  can_change_race=(current_tool->function_press == put_unit);
 } /* tool_toggled() */


void cut(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(5));
 } /* cut() */

void copy(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(6));
 } /* copy() */

void paste(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(7));
 } /* paste() */

void tile(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(0));
 } /* tile() */

void line(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(1));
 } /* line() */

void rectangle(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(2));
 } /* rectangle() */

void special(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(3));
 } /* special() */

void startpos(void)
 {
  tool_toggled(NULL, GINT_TO_POINTER(4));
 } /* startpos() */


/*****************************************************************************
hide the toplevel window upon wm's window delete/destroy
*****************************************************************************/
int destroyWindow(GtkWidget *widget, GdkEvent *ev, gpointer data)
 {
  toggle_tool_off();
  return TRUE;
 } /* destroyWindow() */


/*****************************************************************************
show the toolbox window
*****************************************************************************/
void show_tools(void)
 {
   if(tools_shown)
     gtk_widget_hide(GTK_WIDGET(toolwin));
   else
     gtk_widget_show(GTK_WIDGET(toolwin));
   tools_shown=(!tools_shown);
 } /* show_tools() */


/*****************************************************************************
toggle tooltips on and off
*****************************************************************************/
void show_tips(void)
 {
   if(tooltips_enabled)
    {
     gtk_tooltips_disable(tooltips);
     tooltips_enabled=FALSE;
    }
   else
    {
     gtk_tooltips_enable(tooltips);
     tooltips_enabled=TRUE;
    }
 } /* show_tips() */
