#include "draw.h"
#include "tiles.h"

#include <map.h>

#include <stdlib.h>
#include <stdio.h>

/* the clipboard structure */
typedef struct _undo
 {
  gint x, y;
  gint width, height;
  struct tile *tiles;
  struct map_position start_positions[R_LAST];
  struct _undo *prev;
 } UndoBuffer;

/* the undo buffer */
UndoBuffer *undo=NULL;


/*****************************************************************************
initialize the undo buffer
*****************************************************************************/
void init_undo(void)
 {
  UndoBuffer *temp;

  /* remove all existent undo buffers */
  while(undo!=NULL)
   {
    temp=undo;
    undo=undo->prev;
    if(temp->tiles!=NULL)
      free(temp->tiles);
    free(temp);
   }

 } /* init_undo() */


/*****************************************************************************
fill the undo buffer with the values about to change
*****************************************************************************/
void prime_undo(gint xstart, gint ystart, gint xend, gint yend)
 {
  gint i, x, y;
  UndoBuffer *prev;

  prev=undo;
  if(!(undo=(UndoBuffer *)malloc(sizeof(UndoBuffer))))
   {
    fprintf(stderr, "malloc failed in undo\n");
    exit(1);
   }
  undo->prev=prev;

  undo->x=xstart;
  undo->y=ystart;
  undo->width=xend-xstart+1;
  undo->height=yend-ystart+1;

  if(!(undo->tiles=(struct tile*)malloc((undo->width)*(undo->height)*
     sizeof(struct tile))))
   {
    fprintf(stderr, "malloc failed in undo\n");
    exit(1);
   }

  /* copy the rectangle into the undo buffer */
  i=0;
  for(x=xstart; x<xend+1; x++)
    for(y=ystart; y<yend+1; y++)
      memcpy(undo->tiles+(i++), map_get_tile(x,y), sizeof(struct tile));

  for(i=0;i<R_LAST;i++)
   {
    undo->start_positions[i].x=map.start_positions[i].x;
    undo->start_positions[i].y=map.start_positions[i].y;
   }

 } /* prime_undo() */


/*****************************************************************************
copy the undo buffer back into place
*****************************************************************************/
void do_undo(void)
 {
  gint i, x, y;
  UndoBuffer *prev;

  if(undo==NULL)
    return;

  /* copy the rectangle from the undo buffer onto the map*/
  i=0;
  for(x=undo->x; x<undo->x+undo->width; x++)
    for(y=undo->y; y<undo->y+undo->height; y++)
      memcpy(map_get_tile(x,y), undo->tiles+(i++), sizeof(struct tile));

  for(x=undo->x-1; x<=undo->x+undo->width; x++)
    for(y=undo->y-1; y<=undo->y+undo->height; y++)
      draw_tile(GTK_WIDGET(da), x, y, NULL);

  for(i=0;i<R_LAST;i++)
   {
    x=map.start_positions[i].x;
    y=map.start_positions[i].y;
    map.start_positions[i].x=undo->start_positions[i].x;
    map.start_positions[i].y=undo->start_positions[i].y;
    /* erase the old start position from the display */
    draw_tile(GTK_WIDGET(da), x, y, NULL);
   }

  prev=undo->prev;
  if(undo->tiles!=NULL)
    free(undo->tiles);
  free(undo);
  undo=prev;

  refresh_screen();
 } /* undo() */
